'''
Description: newly observed domains. Create a dictionary in memory. Summerize
the data for the data range specified
Author: @nabeelxy 
Usage: python cert2dom.py YYYYMMDD YYYYMMDD
'''

import ast
import os
import sys
from datetime import timedelta

import pandas as pd

from commons.utils.date_tools import dt2str, ts2dt
from commons.utils.domain_tools import get_2ld

BASE_CT = "/export/sec04/phishing/data"


def get_domains(startdate, enddate, isapex=False):
    s_date = ts2dt(startdate.strip(), "%Y%m%d")
    e_date = ts2dt(enddate.strip(), "%Y%m%d")

    delta = e_date - s_date
    domains = dict()
    count = 0
    for i in range(delta.days + 1):
        curr_date = s_date + timedelta(i)
        name = "certs_" + dt2str(curr_date, "%Y%m%d") + ".log"
        certs = open(os.path.join(BASE_CT, name), "r")
        for cert in certs:
            cert = ast.literal_eval(cert.strip())
            seen = int(cert['data']['seen'])
            for domain in cert['data']['leaf_cert']['all_domains']:
                count += 1
                if domain.startswith("*."):
                    domain = domain[2:]
                domain = domain.strip()
                if domain == "" or len(domain) == 0:
                    continue

            if isapex:
                try:
                    domain = get_2ld(domain)
                except:
                    print("Unable to get 2ld: {}".format(domain))
                    pass
                    continue

            if domain in domains:
                if seen < domains[domain][0]:
                    domains[domain][0] = seen
                if seen > domains[domain][1]:
                    domains[domain][1] = seen
                domains[domain][2] += 1
            else:
                # firstseen, lastseen, count
                domains[domain] = [seen, seen, 1]
    # print("all = {}".format(count))
    # print("processed = {}".format(len(domains)))
    return domains


def save2file(filename, domains):
    df = pd.DataFrame(domains.items())
    df.to_csv(filename, index=False, header=False)


if __name__ == "__main__":
    startd = sys.argv[1]  # YYYYMMDD
    endd = sys.argv[2]  # YYYYMMDD
    outf = sys.argv[3]
    save2file(outf, get_domains(startd, endd))



import tarfile
import json
import glob
import sys
import os
import time
from urllib.parse import urlparse
from datetime import date, timedelta, datetime


def extract_vt(fi, outf):
    fd = open(outf, "w+")
    file_obj = open(fi)	

    for line in file_obj:
        try:
            url_obj = json.loads(line)
            url = url_obj["url"]
            scan_t = url_obj["scan_date"]
            scan_id = url_obj["scan_id"]
            positives = int(url_obj["positives"])
            ip = "NONE"
            try:
                ip = url_obj["additional_info"]["resolution"]
            except:
                pass
            avs = ""
            if positives > 0:
                scans = url_obj['scans']
                for av, result in scans.items():
                    if result["detected"] == True:
                        if len(avs) > 0:
                            avs += '#'
                        avs += av + ':' + result['result']
            fd.write("{}##{}##{}##{}##{}##{}\n".format(url, scan_t, scan_id, positives, avs, ip))
        except:
            pass
            continue
    file_obj.close()
    fd.close()

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print ("Usage: {} <vt_report_file> <output file>".format(sys.argv[0]))
        exit()
    extract_vt(sys.argv[1], sys.argv[2])



import vt_private as vtp
import sys
import json
from urllib.parse import urlparse
import time

SPEED = 0.1

def scan_file(domain_file, outfile, keyindex):
    of = open(outfile, "w+")
    with open(domain_file, encoding = 'utf-8') as df:
        for line in df:
            time.sleep(SPEED)
            line = line.strip()
            if line.startswith("http"):
                domain = line
            else: #if the protocol is not specified, use https
                domain = "https://" + line
            of.write("{}\n".format(json.dumps(vtp.scan(domain, keyindex))))
    of.close()

def report_file(scanid_file, outfile, keyindex):
    of = open(outfile, "w+")
    with open(scanid_file) as sf:
        for str in sf:
            json_obj = json.loads(str)
            try:
                time.sleep(SPEED)
                of.write("{}\n".format(json.dumps(vtp.getReport(json_obj['scan_id'], keyindex))))
            except KeyError as e:
                pass
    of.close()

def process_vt_results(results_file, outfile):
    of = open(outfile, "w+")
    with open(results_file) as rf:
        for str in rf:
            #str = str.replace('\'', '"').replace("True", "\"True\"").replace("False", "\"False\"")
            #print(str)
            json_obj = json.loads(str)
            of.write("{} {}\n".format(urlparse(json_obj['url']).netloc, json_obj['positives']))
    of.close()

domainfile = sys.argv[1]
outputfile = sys.argv[2]
keyindex = int(sys.argv[3])
scan_file(domainfile, outputfile, keyindex)
#time.sleep(300)
report_file(outputfile, outputfile + ".report", keyindex)



from pygeoip import GeoIP, MEMORY_CACHE

class MyGeo:
    def __init__(self, maxmind_city_file):
        self.gi = GeoIP(maxmind_city_file, MEMORY_CACHE)

    def getCountry(self, ip):
        ret = self.gi.record_by_addr(ip)
        if ret != None:
            return ret["country_name"]
        return None

    def getLatLon(self, ip):
        ret = self.gi.record_by_addr(ip)
        if ret != None:
            return [ret["latitude"], ret["longitude"]]
        return None

class MyASN:
    def __init__(self, asn_file):
        self.gi = GeoIP(asn_file, MEMORY_CACHE)

    def getasn(self, ip):
        as_str = self.gi.org_by_addr(ip)
        if as_str != None:
            arr = as_str.split()
            if len(arr) > 0:
                return arr[0].strip()
        return as_str

    def getorg(self, ip):
        as_str = self.gi.org_by_addr(ip)
        if as_str != None:
            i = as_str.find(' ')
            if i != -1:
                return as_str[i+1:].strip()

    def getasn_dom(self, domain):
        as_str = self.gi.asn_by_name(domain)
        return as_str



import requests

'''
Description: Wrappers to connect to torguard proxies
Author: @nabeelxy
'''

# List of paid proxies listed in torguard as of 20191008
proxy_pnts = ['br', 'chil', 'ca', 'vanc.ca.west', 'cr', 'mx', 'atl.east.usa', 'la.west.usa', 'fl.east.usa',
              'dal.central.usa', 'nj.east.usa', 'ny.east.usa', 'chi.central.usa', 'lv.west.usa', 'sf.west.usa',
              'sa.west.usa', 'aus', 'bl', 'bg', 'bul', 'cp', 'czech', 'den', 'fin', 'fr', 'gr', 'gre', 'hg', 'ice',
              'ire', 'it', 'lv', 'lux', 'md', 'nl', 'no', 'pl', 'por', 'ro', 'slk', 'sp', 'swe', 'swiss', 'turk',
              'ukr', 'uk', 'au', 'hk', 'jp', 'sk', 'my', 'nz', 'singp', 'tw', 'thai', 'egy', 'in', 'isr', 'isr2', 'za',
              'uae']


def get_proxy(proxy_index, username, password):
    return "{0}:{1}@{2}.torguardvpnaccess.com:6060".format(username, password, proxy_pnts[proxy_index])


def get_working_proxies(current_proxy_pnts, username=None, password=None):
    url = 'https://httpbin.org/ip'
    new_proxy_pnts = list()
    for proxy_index in range(0, len(current_proxy_pnts)):
        proxy = get_proxy(proxy_index, username, password)
        try:
            r = requests.get(url, proxies={'http': proxy, 'https': proxy})
            if r.status_code == 200:
                new_proxy_pnts.append(current_proxy_pnts[proxy_index])
            else:
                print("Error: proxy {}".format(current_proxy_pnts[proxy_index]))
        except Exception as e:
            print(e)
            print("Timed out proxy {}".format(current_proxy_pnts[proxy_index]))
            pass
    return new_proxy_pnts


def proxy_exe(url, proxy_index, username=None, password=None, method="get", params=None, data=None, headers=None):
    proxy = get_proxy(proxy_index, username, password)
    try:
        if method == "get":
            res = requests.get(url, proxies={'http': proxy, 'https': proxy}, headers=headers, params=params)
        elif method == "post":
            res = requests.post(url, data=data, proxies={'http': proxy, 'https': proxy}, headers=headers, params=params)
        else:
            print("Unsupported HTTP method: {}".format(method))
            return None
        if res.status_code == 200:
            return res
        else:
            print("Error: proxy {}".format(proxy_pnts[proxy_index]))
    except Exception as e:
        print(e)
        print("Time out: proxy {}".format(proxy_pnts[proxy_index]))
        pass
    return None



#Obtaining the authoritative name server
import dns.resolver
import re

domain = 'co.uk'

response = dns.resolver.query(domain, 'SOA')

if response.rrset is not None:
    pattern= r'(%s)\.\s(\d{1,})\s(\w+)\sSOA\s(.*?)\.\s(.*?)\.\s(\d{1,})\s(\d{1,})\s(\d{1,})\s(\d{1,})\s(\d{1,})' % domain
    match = re.match(pattern, str(response.rrset))
    m_name, ttl, class_, ns, email, serial, refresh, retry, expiry, minim = match.groups()

output ='''
Main Name In Zone: {a},
Cache TTL: {b},
Class: {c},
Authoritive NS: {d},
Email Address: {e},
Last Change: {f},
Retry In Secs: {g},
Expiry: {h},
Slave Cache In Sec: {i}
'''.format(a = m_name, b = ttl, c = class_, d = ns, e = str(email).replace('\\', ''), f = serial, g = retry, h = expiry, i = minim)

print output

response = dns.resolver.query(domain, 'SOA')
print response.rrset



from pysafebrowsing import SafeBrowsing
import pandas as pd
import sys

keys = ['AIzaSyCXUW2uCTHoSyLjAIEKzNRPUIXCfG3MScU', #0 digestize
        'AIzaSyCtXkzebCAfuISFqAog9pep_sSvY1-9SWY', #1 minhaj
        'AIzaSyAoA6JzjEYv3vqBqiJT4BS621sBJmodiao'] #2



infile = sys.argv[1]
outfile = sys.argv[2]
index = int(sys.argv[3])

gsb = SafeBrowsing(keys[index])

urls = []
with open(infile) as inf:
    for line in inf:
        urls.append(line.strip())

res = gsb.lookup_urls(urls)

if res != None:
    drop_list = []
    for key, values in res.items():
        try:
            if res[key]['malicious'] == False:
                drop_list.append(key)
        except KeyError as e:
            print("Key error: URL: {}".format(key))
            pass

    for url in drop_list:
        del res[url]

    res2 = []
    for key, values in res.items():
        values['url'] = key
        res2.append(values)

    res_df = pd.DataFrame().from_records(res2)
    res_df.to_csv(outfile, index = False)




'''
Description: Expanding on domain or IP based on domain-ip resolution graph
Author: @nabeelxy 
'''

import pdns
import sys
import os
from datetime import datetime

def get_firstseen_for_domains(domain_file, outfile, lastseen):
  df = open(domain_file)
  of = open(outfile, "w+")
  for domain in df:
    domain = domain.strip()
    entries = pdns.pdns_domain(domain, None, None, None, lastseen)
    if entries != None:
      firstseen = pdns.get_first_seen_date(entries)
      of.write("{}###{}\n".format(domain, firstseen))
  of.close()
  df.close() 

def get_firstseen_lastseen_for_domains(domain_file, outfile):
  df = open(domain_file)
  of = open(outfile, "w+")
  for domain in df:
    domain = domain.strip()
    entries = pdns.pdns_domain(domain, None, None, None, None)
    if entries != None:
      [firstseen, lastseen] = pdns.get_first_last_seen_date(entries)
      of.write("{} {} {} {}\n".format(domain, firstseen, lastseen, (lastseen - firstseen)/86400))
  of.close()
  df.close() 

#Given a list of domains, get all IPs seen after a certain time
def get_ips_for_domains(domain_file, outfile, lastseen, limit):
  df = open(domain_file)
  of = open(outfile, "w+")
  for domain in df:
    domain = domain.strip()
    entries = pdns.pdns_domain(domain, None, None, None, lastseen, limit)
    if entries != None:
      for entry in entries:
        for ip in entry["rdata"]:
          of.write("{} {}\n".format(domain, ip))
  of.close()
  df.close() 

# get the ips for a given set of domains for time period between st (start time) and et (end time)
# include_outside captures those IPs that were seen before the start time and after the end time,
# not within the time interval (it may or may not have appeared within the time interval. The default
# value is false to exlude by default.
def get_ips_for_time_int(domain_file, outfile, st, et, include_outside = False, limit = 1000):
  df = open(domain_file)
  of = open(outfile, "w+")
  for domain in df:
    domain = domain.strip()
    entries = pdns.pdns_domain(domain, None, None, None, None, limit)
    if entries != None:
      for entry in entries:
        time_first = entry['time_first']
        time_last = entry['time_last']
        #The api returns only if the last seen is within that week. In other words,
        #if the last seen advnaces, it does not output the domain-ip pair when
        #time fenced with last seen. In this, we get all domain-ip pairs and
        #filter those that are likely to be seen on the given time frame. Note
        #that this is only an estimation. Some domain-ip pairs may or may not have
        #been seen on the given time interval.

        #conditions
        #first seen is within the interval
        #last seen is within the interval
        #while not seen in the interval, but seen before and after + include check
        if (time_first >= st and time_first <= et) or \
           (time_last >= st and time_last <= et) or \
           (include_outside and time_first < st and time_last > et): 
          for ip in entry["rdata"]:
            of.write("{} {}\n".format(domain, ip))
  of.close()
  df.close() 

def get_ips_for_time_int2(domain, st, et, include_outside = False, limit = 1000):
  new_entries = None
  entries = pdns.pdns_domain(domain, None, None, None, None, limit)
  if entries != None:
    for entry in entries:
      time_first = entry['time_first']
      time_last = entry['time_last']
      if (time_first >= st and time_first <= et) or \
         (time_last >= st and time_last <= et) or \
         (include_outside and time_first < st and time_last > et): 
        if new_entries == None:
          new_entries = list()
        new_entries.append(entry)
  return new_entries

#given a twold, get the likely seen authoritative NS entries for the twold
#given a twold, get the likely seen authoritative NS entries for the twold
def get_ns_for_time_int(twold, st, et, include_outside = False, limit = 100):
    entries = pdns.pdns_ns(twold, None, None, None, None, limit)
    new_entries = None
    #TODO - refactor this code
    if entries != None:
        for entry in entries:
            time_first = entry['time_first']
            time_last = entry['time_last']
            if (time_first >= st and time_first <= et) or \
                (time_last >= st and time_last <= et) or \
                (include_outside and time_first < st and time_last > et): 
                if new_entries == None:
                    new_entries = list()
                new_entries.append(entry)
    return new_entries

#given an IP, get the domains likely seen resolving to that IP during a given time period
def get_domains_for_time_int(ip, st, et, include_outside = False, limit = 5000):
    entries = pdns.pdns_ip(ip, None, None, None, None, limit)
    new_entries = None
    if entries != None:
        for entry in entries:
            time_first = entry['time_first']
            time_last = entry['time_last']
            if (time_first >= st and time_first <= et) or \
                (time_last >= st and time_last <= et) or \
                (include_outside and time_first < st and time_last > et): 
                if new_entries == None:
                    new_entries = list()
                new_entries.append(entry)
    return new_entries

#seen after a certain timestamp
def get_domains_for_ips(ip_file, outfile, lastseen, limit = None):
  ipf = open(ip_file)
  of = open(outfile, "w+")
  for ip in ipf:
    ip = ip.strip()
    entries =  pdns.pdns_ip(ip, None, None, None, lastseen, limit)
    if entries != None:
      for entry in entries:
        of.write("{} {}\n".format(entry['rrname'], ip))
  of.close()
  ipf.close()

def get_set_(inputf, index):
  ds = set()
  with open(inputf) as inf:
    for line in inf:
      ds.add(line.strip().split()[index].strip("."))
  return ds

def get_domainset(inputf):
  return get_set_(inputf, 0)

def get_ipset(inputf):
  return get_set_(inputf, 1)        

def get_domains_for_time_int_set(ipset, outf, st, et, outside, limit):
  of = open(outf, "w+")
  for ip in ipset:
    ip = ip.strip()
    entries = get_domains_for_time_int(ip, st, et, outside, limit)
    if entries != None:
      for entry in entries:
        of.write("{} {}\n".format(entry['rrname'].strip("."),ip))
  of.close()

def get_ips_for_time_int2_set(domainset, outf, st, et, outside, limit):
  of = open(outf, "w+")
  for domain in domainset:
    domain = domain.strip()
    entries = get_ips_for_time_int2(domain, st, et, outside, limit)  
    if entries != None:
      for entry in entries:
        for ip in entry['rdata']:
          of.write("{} {}\n".format(domain,ip))
  of.close()

def expansion_generic(exp_type, start_time, end_time, limt, level, infile, outfile):
  ipset = set()
  domainset = set()
  turn = 0
  if exp_type == "ip":
    with open(infile) as inf:
      for ip in inf:
        ipset.add(ip.strip())
    get_domains_for_time_int_set(ipset, outfile+"_1", start_time, end_time, True, limit)
  elif exp_type == "domain":
    turn = 1
    with open(infile) as inf:
      for domain in inf:
        domainset.add(domain.strip())
    get_ips_for_time_int2_set(domainset, outfile + "_1", start_time, end_time, True, limit)
  else:
    print("Invalid expansion type: {}".format(exp_type))
    return
  
  if level == 1:
    return 

  prev_domainset = domainset
  prev_ipset = ipset
  print("Level: {} #IPs: {}".format(1, len(ipset)))
  print("Level: {} #Domains: {}".format(1, len(domainset)))

  for l in range(2, level+1):
    if l%2 == turn: #domain lookup
      domainset = get_domainset(outfile + "_" + str(l - 1))
      if prev_domainset != None:
        domainset_temp = domainset - prev_domainset
        prev_domainset = prev_domainset | domainset
        domainset = domainset_temp
      print("Level: {} #Domains: {}".format(l, len(domainset)))
      if len(domainset) == 0:
        print("Done")
        return 
      get_ips_for_time_int2_set(domainset, outfile + "_" + str(l), start_time, end_time, True, limit)
    else: #ip lookup
      ipset = get_ipset(outfile + "_" + str(l - 1))
      if prev_ipset != None:
        ipset_temp = ipset - prev_ipset
        prev_ipset = prev_ipset | ipset
        ipset = ipset_temp
      print("Level: {} #IPs: {}".format(l, len(ipset)))
      if len(ipset) == 0:
        print("Done")
        return
      get_domains_for_time_int_set(ipset, outfile + "_" + str(l), start_time, end_time, True, limit)

#Get the duration of a given list of domains
def get_duration(inputf, outputf):
  inf = open(inputf)
  outf = open(outputf, "w+")

  for domain in inf:
    domain = domain.strip()
    entries = pdns.pdns_domain(domain)
    if entries != None:
      [firstseen, lastseen] = pdns.get_first_last_seen_date(entries)
      outf.write("{} {} {} {}\n".format(domain, firstseen, lastseen, float(lastseen - firstseen)/86400))
  outf.close()
 
#===================================================================================================
if len(sys.argv) < 4:
  print("Usage: {} [domain_lookup|ip_lookup|duration|firstseen|firstseen_lastseen|domain_lookup_int|ip_expansion|domain_expansion] input_file out_file")
  exit()

infile = sys.argv[2].strip()
outfile = sys.argv[3].strip()
method = sys.argv[1].strip()

if method == "domain_lookup":
  if len(sys.argv) < 5:
    print("Missing 4th parameter: lastseen_after")
    exit()
  lastseen = sys.argv[4]
  if lastseen == "None":
    lastseen = None
  limit = None
  if len(sys.argv) > 5:
    limit = int(sys.argv[5])
  get_ips_for_domains(infile, outfile, lastseen, limit)
elif method == "ip_lookup":
  if len(sys.argv) < 5:
    print("Missing 4th parameter: lastseen_after")
    exit()
  lastseen = sys.argv[4]
  if lastseen == "None":
    lastseen = None
  limit = None
  if len(sys.argv) > 5:
    limit = int(sys.argv[5])
  get_domains_for_ips(infile, outfile, lastseen, limit)
elif method == "firstseen":
  if len(sys.argv) < 5:
    print("Missing 4th parameter: lastseen_after")
    exit()
  lastseen = sys.argv[4]
  if lastseen == "None":
    lastseen = None
  get_firstseen_for_domains(infile, outfile, lastseen)
elif method == "firstseen_lastseen":
  get_firstseen_lastseen_for_domains(infile, outfile)
elif method == "domain_lookup_int":
  if len(sys.argv) < 6:
    print("Missing 4th and 5th parameter: start_time(YYYYMMDD HH:MM) end_time(YYYYMMDD HH:MM)")
    exit()
  os.environ["TZ"] = "UTC"
  start_time = int(datetime.strptime(sys.argv[4], '%Y%m%d %H:%M').strftime('%s'))
  end_time = int(datetime.strptime(sys.argv[5], '%Y%m%d %H:%M').strftime('%s'))
  #the last parameter includes those IPs seen before and after the interval (not within)
  get_ips_for_time_int(infile, outfile, start_time, end_time, True)
elif method == "ip_expansion":
  if len(sys.argv) < 8:
    print("Missing 4th, 5th, 6th and 7th parameters: start_time(YYYYMMDD HH:MM) end_time(YYYYMMDD HH:MM) limit level")
    exit()
  os.environ["TZ"] = "UTC"
  start_time = int(datetime.strptime(sys.argv[4], '%Y%m%d %H:%M').strftime('%s'))
  end_time = int(datetime.strptime(sys.argv[5], '%Y%m%d %H:%M').strftime('%s'))
  limit = int(sys.argv[6])
  level = int(sys.argv[7])
  if level < 1:
    print("Level is {}. But it must be greater than or equal to 1".format(level))
    exit()
  expansion_generic("ip", start_time, end_time, limit, level, infile, outfile)
elif method == "domain_expansion":
  if len(sys.argv) < 8:
    print("Missing 4th, 5th, 6th and 7th parameters: start_time(YYYYMMDD HH:MM) end_time(YYYYMMDD HH:MM) limit level")
    exit()
  os.environ["TZ"] = "UTC"
  start_time = int(datetime.strptime(sys.argv[4], '%Y%m%d %H:%M').strftime('%s'))
  end_time = int(datetime.strptime(sys.argv[5], '%Y%m%d %H:%M').strftime('%s'))
  limit = int(sys.argv[6])
  level = int(sys.argv[7])
  if level < 1:
    print("Level is {}. But it must be greater than or equal to 1".format(level))
    exit()
  expansion_generic("domain", start_time, end_time, limit, level, infile, outfile)
elif method == "duration":
  get_duration(infile, outfile)
else:
  print("Unsupported operation")




from selenium import webdriver
import pandas as pd
from selenium.common.exceptions import NoSuchElementException, TimeoutException
import tldextract
from urllib.parse import urlparse
import time
import random
from bs4 import BeautifulSoup
import sys
import os
import math
import glob

USER_AGENTS = [
"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36",
"Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36",
"Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36",
"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36",
"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36",
"Mozilla/5.0 (iPhone; CPU iPhone OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/83.0.4147.71 Mobile/15E148 Safari/604.1",
"Mozilla/5.0 (iPad; CPU OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/83.0.4147.71 Mobile/15E148 Safari/604.1",
"Mozilla/5.0 (iPod; CPU iPhone OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/83.0.4147.71 Mobile/15E148 Safari/604.1",
"Mozilla/5.0 (Linux; Android 10) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Mobile Safari/537.36",
"Mozilla/5.0 (Linux; Android 10; SM-A205U) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Mobile Safari/537.36"
]

def get_2ld(fqdn, only_tld = False):
    try:
        ext = tldextract.extract(fqdn)
        if len(ext.domain) == 0 or len(ext.suffix) == 0:
            return None
        else:
            if only_tld:
                return ext.suffix
            else: 
                return ext.domain + "." + ext.suffix
    except Exception as e:
        pass
    return None

def get_fqdn(url):
    try:
        uri = urlparse(url)
        if uri != None:
            return uri.netloc
    except Exception as e:
        pass

def start_browser(agent_index):
    #brew cask install chromedriver
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument("--incognito")
    chrome_options.add_argument('user-agent="{}"'.format(USER_AGENTS[agent_index]))
    browser = webdriver.Chrome(executable_path=r"chromedriver", options=chrome_options)

    browser.set_page_load_timeout(10)

    return browser

def fetch_url(browser, url):
    try:
        browser.get(url)
        return browser.page_source
    except Exception as e:
        return "<html>Error: {}</html>".format(e) 

def close_browser(browser):
    browser.close()

def save_links(response, outfile):
    f1 = open(outfile, "a", encoding = 'utf-8')
    f1.write(response)
    f1.close()

def parse_page(outfile):
    f2 = open(outfile + "_parsed", "w", encoding = 'utf-8')
    soup = BeautifulSoup(open(outfile).read(), features = "html.parser")
    table = soup.find("table")
    if table != None:
        trs = table.find_all("tr")
        for tr in trs:
            tds = tr.find_all("td")
            row = []
            for td in tds:
                row.append(td.text)
            f2.write("{}\n".format("#####".join(row)))
    f2.close()

def extract_url(res):
    soup = BeautifulSoup(res, features = "html.parser")
    span = soup.find("span", {"style":"word-wrap:break-word;"})
    if span != None:
        b = span.find("b")
        if b != None:
            return b.text
    return None

def collect_urls(parsed_file, last_phishid):
    pf = open(parsed_file, encoding = 'utf-8')
    outf = open(parsed_file + "_fullurls", "w", encoding = 'utf-8')
    agent_index = math.floor(random.uniform(0, len(USER_AGENTS)))
    browser = start_browser(agent_index)
    highest_phishid = -1
    for line in pf:
        wt = random.uniform(3, 6)
        arr = line.split("#####")
        if len(arr) != 5:
            continue
        try:
            phishid = int(arr[0])
            if phishid > highest_phishid:
                highest_phishid = phishid
        except:
            pass
            continue
        #only collect those phishing urls that are not collected previously
        if phishid <= last_phishid:
            continue

        url = arr[1]
        if "SPAM-TO-BE-DELETED" in url:
            continue
        #... means that the full URL is not availble
        if "..." in url:
            time.sleep(wt)
            lookup = "http://phishtank.org/phish_detail.php?phish_id={}".format(phishid)
            res = fetch_url(browser, lookup)
            if res != None:
                full_url = extract_url(res)
            print("{}#####{}".format(phishid, full_url))
            outf.write("{}#####{}\n".format(phishid, full_url))
    outf.close()
    close_browser(browser)
    return highest_phishid
    
def save_page(outfile, pages, phishidfile):
    agent_index = math.floor(random.uniform(0, len(USER_AGENTS)))
    browser = start_browser(agent_index)
    last_phishid = int(open(phishidfile).read().strip())
    for page in range(0, pages):
        wt = random.uniform(5, 10)
        try:
            #Try not to annnoy Google, with a random short wait
            url = "http://phishtank.org/phish_search.php?page={}".format(page)
            new_filename = outfile + "_" + str(page) + "_raw"
            save_links(fetch_url(browser, url), new_filename)
            parse_page(new_filename)
            phishid = -1
            try:
                phishid = collect_urls(new_filename + "_parsed", last_phishid)
            except:
                print("url parsing error")
                print(e)
                pass

            #write the latest phish id for the current lookup
            if page == 0:
                if phishid == -1:
                    print("ERROR: Highest phish id cannot be -1")
                else:
                    pid_f = open(phishidfile, "w")
                    pid_f.write("{}".format(phishid))
                    pid_f.close()
            process_urls(new_filename + "_parsed", last_phishid)
            #don't collect already collected ones
            if phishid <= last_phishid:
                break
            time.sleep(wt)
        except Exception as e:
            print(e)
            pass
    close_browser(browser)

#parse page and collect truncated URLs
def collect_all_urls(outfile):
    for filepath in glob.iglob(outfile + '_*_raw'):
        parse_page(filepath)
        collect_urls(filepath + "_parsed")

def process_urls(parsed_file, last_phishid):
    pf = open(parsed_file, encoding = 'utf-8')
    uf = open(parsed_file + "_fullurls", encoding = 'utf-8')
    ff = open(parsed_file + "_processed", "w", encoding = 'utf-8')

    #load URLs
    full = dict()
    for line in uf:
        arr = line.split("#####")
        if len(arr) != 2:
            continue
        full[arr[0]] = arr[1].strip()

    #process each row
    for line in pf:
        arr = line.strip().split("#####")
        if len(arr) != 5:
            continue
        phishid = arr[0]

        #don't collect urls that are already collected
        try:
            if int(phishid) <= last_phishid:
                continue
        except:
            pass
            continue

        url = arr[1]
        if len(url.strip()) == 0:
            continue
        submitted = "None"
        i = url.find("added on")
        if i != -1:
            submitted = url[i + 9 :]
        if "SPAM-TO-BE-DELETED" in url:
            continue
        #... means that the full URL is not availble
        if "..." in url and phishid in full:
            url = full[phishid]
        else:
            if i != -1:
                url = url[:i]
        if "by" in arr[2]:
            arr[2] = arr[2][3:]

        ff.write("{}\n".format("#####".join([phishid, url, submitted, arr[2], arr[3], arr[4]])))
    ff.close()

#produce a proper csv of the phishtank file    
#def process_all_urls(outfile):
#    for filepath in glob.iglob(outfile + '_*_raw_parsed'):
#        process_urls(filepath)

def aggregate_processed(outfile):
    bdir = os.path.dirname(outfile)
    os.system("cat {}_*_raw_parsed_processed >> {}_all".format(outfile, outfile))
    os.system("mv {}_*_raw_parsed_processed {}/tmp".format(outfile, bdir))
    os.system("cat {}_*_raw_parsed >> {}_raw_parsed".format(outfile, outfile))
    os.system("mv {}_*_raw_parsed {}/tmp".format(outfile, bdir))
    os.system("cat {}_*_raw >> {}_raw".format(outfile, outfile))
    os.system("mv {}_*_raw {}/tmp".format(outfile, bdir))
    os.system("cat {}_*_raw_parsed_fullurls >> {}_raw_parsed_fullurls".format(outfile, outfile))
    os.system("mv {}_*_raw_parsed_fullurls {}/tmp".format(outfile, bdir))

#later check phishing urls
def check_phish(url):
    full_url = "https://checkurl.phishtank.com/checkurl/index.php?url={}".format(url)
    #TODO - implement

if __name__ == "__main__":
    option = sys.argv[1]
    if option == "page":
        #generic file prefix, number of pages, phish_id page
        save_page(sys.argv[2], int(sys.argv[3]), sys.argv[4])
        aggregate_processed(sys.argv[2])



'''
Description: VT profile utility functions
Author: @nabeelxy
'''

import re
import string

def get_num(char):
  numbers = dict()
  numbers['0'] = "zero"
  numbers['1'] = "one"
  numbers['2'] = "two"
  numbers['3'] = "three"
  numbers['4'] = "four"
  numbers['5'] = "five"
  numbers['6'] = "six"
  numbers['7'] = "seven"
  numbers['8'] = "eight"
  numbers['9'] = "nine"
  return numbers[char]

def get_table_old(domain):
  pattern_char = re.compile("^[a-z]")
  pattern_digit = re.compile("^[0-9]")
  if (len(domain) > 0):
    lowerdomain = domain.lower()
  if (pattern_char.search(lowerdomain)):
    return lowerdomain[0] + "_vt"
  elif (pattern_digit.search(lowerdomain)):
    return get_num(lowerdomain[0]) +"_vt"
  else:
    return "special_vt"
#given the domain name, return the table name to use
def get_table(domain):
  pattern_char = re.compile("^[a-z]")
  pattern_digit = re.compile("^[0-9]")
  if (len(domain) > 0):
    lowerdomain = domain.lower()
    if (pattern_char.search(lowerdomain)):
      #alpha-numeric start
      second_level = "rest"
      if (len(domain) > 1 and string.ascii_lowercase.find(lowerdomain[1]) > -1):
        second_level = lowerdomain[1]

      return lowerdomain[0] + "_" + second_level + "_vt"
    elif (pattern_digit.search(lowerdomain)):
      second_level = "rest"
      if (len(domain) > 1 and string.digits.find(lowerdomain[1]) > -1):
        second_level = get_num(lowerdomain[1])

      return get_num(lowerdomain[0]) + "_" + second_level +"_vt"
    else:
      #special character start
      return "special_vt"

def get_cursor_key(domain):
  pattern_char = re.compile("^[a-z0-9]")
  if (len(domain) > 0):
    lowerdomain = domain.lower()
    if (pattern_char.search(lowerdomain)):
      second_key = "rest"
      if (len(domain) > 1 and pattern_char.search(lowerdomain[1]) and 
         ((string.ascii_lowercase.find(lowerdomain[0]) > -1 and string.ascii_lowercase.find(lowerdomain[1]) > -1) or 
         (string.digits.find(lowerdomain[0]) > -1 and string.digits.find(lowerdomain[1]) > -1))):
        second_key = lowerdomain[1]
      #alpha-numeric start
      return lowerdomain[0] + "_" + second_key
    else:
      #special character start
      return 'special'

def get_tables():
  tables = list()
  for char in string.ascii_lowercase:
    for char2 in string.ascii_lowercase:
      tables.append(char + "_" + char2 + "_vt")
    tables.append(char + "_rest" + "_vt")
  for dig in string.digits:
    for dig2 in string.digits:
      tables.append(get_num(dig) + "_" + get_num(dig2) + "_vt")
    tables.append(get_num(dig) + "_rest" + "_vt")
  tables.append("special_vt") 
  return tables

def get_prefixes_new():
  prefixes = list()
  for char in string.ascii_lowercase:
    for char2 in string.ascii_lowercase:
      prefixes.append(char + "_" + char2)
    prefixes.append(char + "_rest")
  for dig in string.digits:
    for dig2 in string.digits:
      prefixes.append(dig + "_" + dig2)
    prefixes.append(dig + "_rest")
  prefixes.append("special") 
  return prefixes 

def get_prefixes():
  prefixes = list()
  for char in string.ascii_lowercase:
    prefixes.append(char)
  for dig in string.digits:
    prefixes.append(get_num(dig))
  prefixes.append("special") 
  return prefixes 



from cert_oid import oid_dv, oid_ov, oid_ev, oid_iv

from commons.utils.date_tools import vtdate2epoch


def find_validity_type(cert):
    try:
        cert_policies = cert["attributes"]["extensions"]["certificate_policies"]
        for p in cert_policies:
            if p in oid_dv:
                return 'DV'
            elif p in oid_ov:
                return 'OV'
            elif p in oid_ev:
                return 'EV'
            elif p in oid_iv:
                return 'IV'
    except KeyError as e:
        pass

    try:
        subject_o = cert["attributes"]["subject"]["O"]
        if subject_o != None and "domain control validated" in subject_o.lower():
            return 'DV'
    except KeyError as e:
        pass
        # subject organization is not persent meaning it is not O or E validated
        return 'DV'

    try:
        issuer_o = cert["attributes"]["issuer"]["O"]
        if issuer_o != None and ("domain control validated" in issuer_o.lower() or \
                                 "let's encrypt" in issuer_o.lower()):  # Let's Encrypt only produce DV
            return 'DV'
    except KeyError as e:
        pass

    return 'NA'


# Extract features from the VirusTotal certificates (JSON)
# TODO: consider historical features
def vtcert2csv(vtj):
    features = dict()
    features['subject'] = None
    features['issuer'] = None
    features['creation'] = None
    features['expiration'] = None
    features['san'] = None
    features['type'] = None

    try:
        cert_all = vtj["data"]
        if len(cert_all) == 0:
            return features

        # only consider the latest entry
        cert = cert_all[0]

        try:
            features['subject'] = cert["attributes"]["subject"]["CN"].strip()
        except KeyError as e:
            pass

        try:
            features['issuer'] = cert["attributes"]["issuer"]["O"].strip()
        except KeyError as e:
            pass

        try:
            features['creation'] = vtdate2epoch(cert["attributes"]["validity"]["not_before"].strip())
        except KeyError as e:
            pass

        try:
            features['expiration'] = vtdate2epoch(cert["attributes"]["validity"]["not_after"].strip())
        except KeyError as e:
            pass

        try:
            features['san'] = cert["attributes"]["extensions"]["subject_alternative_name"]
        except KeyError as e:
            pass

        features['type'] = find_validity_type(cert)
    except KeyError as e:
        pass
    return features



import json
import sys
from urllib.parse import urlparse

import pandas as pd


def get_fqdn(url):
    try:
        uri = urlparse(url)
        if uri:
            return uri.netloc
    except Exception as e:
        pass
    return None


def parse_domain_cert(vtfile, outfile):
    of = open(outfile, "w+")
    with open(vtfile) as sf:
        for str in sf:
            json_obj = json.loads(str)
            try:
                for cert in json_obj["data"]:
                    subject = cert["attributes"]["subject"]["CN"].strip()
                    issuer = cert["attributes"]["issuer"]["O"].strip()
                    start = cert["attributes"]["validity"]["not_before"].strip()
                    end = cert["attributes"]["validity"]["not_after"].strip()
                    san = cert["attributes"]["extensions"]["subject_alternative_name"]
                    of.write("{}##{}##{}##{}##{}\n".format(subject, start, end, issuer, san))
            except KeyError as e:
                # print(e)
                pass
    of.close()


def parse_domain_whois(vtfile, outfile):
    records = []
    with open(vtfile) as vf:
        for line in vf:
            json_obj = json.loads(line.strip())
            try:
                for whois in json_obj["data"]:
                    registrant = None
                    registrar = None
                    whoisserver = None
                    creation = None
                    update = None
                    expiration = None
                    ns = None
                    domain = whois["attributes"]["whois_map"]["Domain Name"].lower()
                    try:
                        registrant = whois["attributes"]["registrant_name"].lower()
                    except:
                        registrant = None
                        pass
                    try:
                        registrar = whois["attributes"]["whois_map"]["Registrar"].lower()
                    except:
                        try:
                            registrar = whois["attributes"]["registrar_name"].lower()
                        except:
                            regisrar = None
                    try:
                        creation = whois["attributes"]["whois_map"]["Creation Date"]
                    except:
                        creation = None

                    try:
                        expiration = whois["attributes"]["whois_map"]["Registry Expiry Date"]
                    except:
                        expiration = None

                    try:
                        update = whois["attributes"]["whois_map"]["Updated Date"]
                    except:
                        update = None

                    try:
                        ns = whois["attributes"]["whois_map"]["Name Server"]
                    except:
                        ns = None

                    try:
                        whoiserver = whois["attributes"]["whois_map"]["Registrar WHOIS Server"]
                    except:
                        whoisserver = None

                    # print("registrant country = {}".format(whois["attributes"]["whois_map"]["Registrant Country"]))
                    records.append([domain, registrant, registrar, whoisserver, creation, expiration, ns, update])
            except KeyError as e:
                print(e)
                pass
    df = pd.DataFrame().from_records(records)
    df.to_csv(outfile, index=False, header=False)


def parse_ip_whois(vtfile, outfile):
    of = open(outfile, "w")
    with open(vtfile) as vf:
        for line in sf:
            json_obj = json.loads(line.strip())
            try:
                for whois in json_obj["data"]:
                    print(whois)
            except KeyError as e:
                print(e)
                pass


# before create the dataframe, we need to the attack types and engine names
def get_attacks_engines(rfile):
    rf = open(rfile)
    engines = set()
    attacks = set()
    for line in rf:
        try:
            jobj = json.loads(line.strip())
            scans = jobj["scans"]
            for key, value in scans.items():
                engines.add(key)
                attacks.add(value['result'])
        except KeyError as e:
            pass
    return [attacks, engines]


def parse_domain_reports(rfile, ofile):
    rf = open(rfile)
    [attacks, engine_names] = get_attacks_engines(rfile)
    records = list()

    for line in rf:
        try:
            jobj = json.loads(line.strip())
            url = jobj["url"].strip("/")
            record = dict()
            record["url"] = url
            record["scan_date"] = jobj["scan_date"]
            record["positives"] = jobj["positives"]

            for attack in attacks:
                record[attack] = 0
            for engine in engine_names:
                record[engine] = -2

            scans = jobj["scans"]
            for key, value in scans.items():
                record[value['result']] += 1
                if value['detected']:
                    record[key] = 1
                else:
                    record[key] = 0

                if value['result'] == "unrated site":
                    record[key] = -1

            records.append(record)
        except KeyError as e:
            print(e)
            pass
    df = pd.DataFrame(records)
    df['fqdn'] = df['url'].apply(lambda x: get_fqdn(x) if x != None else x)
    df.to_csv(ofile, header=True, index=False)


def parse_ip_resolutions(rfile, ofile):
    rf = open(rfile)
    records = list()

    for line in rf:
        try:
            jobj = json.loads(line.strip())
            data = jobj["data"]
            for rec in data:
                tmp = []
                attr = rec["attributes"]
                tmp.append(attr["host_name"])
                tmp.append(attr["ip_address"])
                tmp.append(attr["date"])
                records.append(tmp)
        except KeyError as e:
            print(e)
            pass
    df = pd.DataFrame().from_records(records)
    df.to_csv(ofile, header=False, index=False, sep=" ")


if __name__ == "__main__":
    option = sys.argv[1]
    infile = sys.argv[2]
    outfile = sys.argv[3]
    if option == "domain_whois":
        parse_domain_whois(infile, outfile)
    elif option == "domain_report":
        parse_domain_reports(infile, outfile)
    elif option == "ip_resolution":
        parse_ip_resolutions(infile, outfile)
    elif option == "domain_cert":
        parse_domain_cert(infile, outfile)



"""
This python script is used to check urls against google safe browser API. This scripts takes a input file of urls and
check those urls against google safe browser API.
Usage: python gsb_process.py <input file> 
Output: output will a file in same directory as input with name as <input file>_output. Each line of output file will be
 url and status of url e.g True for malicious status and False for Benign status
"""

import json
import sys

import requests

proxy_pnts = ['au', 'melb.au', 'bul', 'cp', 'egy', 'iom', 'isr', 'fin', 'br', 'ca', 'vanc.ca.west', 'frank.gr', 'ice',
              'ire', 'in', 'jp', 'nl', 'uk', 'lon.uk', 'ro', 'ru', 'mos.ru', 'swe', 'swiss', 'bg', 'hk', 'cr', 'hg',
              'ind', 'my', 'thai', 'turk', 'tun', 'mx', 'singp', 'saudi', 'fr', 'pl', 'czech', 'gre', 'it', 'sp', 'no',
              'por', 'za', 'den', 'vn', 'pa', 'sk', 'cn', 'lv', 'lux', 'nz', 'md', 'uae', 'slk', 'fl.east.usa',
              'atl.east.usa', 'ny.east.usa', 'chi.central.usa', 'dal.central.usa', 'la.west.usa', 'lv.west.usa',
              'sa.west.usa', 'nj.east.usa', 'central.usa', 'centralusa', 'west.usa', 'westusa', 'east.usa', 'eastusa']

result = dict()
api_key = ""
final_result = dict()


def main():
    global final_result
    global api_key
    input_urls, url_dict = get_url_list(inpf)
    for key in url_dict:
        api_key = api_keys[key]
        proxy = get_proxy(key)
        proxy_dict = {'http': proxy}
        clientId = "qcri_{}".format(key)
        lookup_urls_all(clientId, url_dict[key], proxy_dict)

    outf = "{0}.gsb".format(inpf)
    with open(outf, 'w+', encoding='utf-8') as f:
        for key in final_result:
            f.write("{} {}\n".format(key, final_result[key]))


def get_url_list(inpf):
    input_urls = set()
    url_dict = {}
    index = 0
    block = 10000  # in practice, this value should be 10000

    with open(inpf, encoding='utf-8') as f:
        for line in f:
            input_urls.add(line.strip())

    print("total unique urls: {}".format(len(input_urls)))
    url_list = list(input_urls)

    for i in range(0, len(url_list), block):
        if index == len(api_keys):
            break
        url_dict[index] = url_list[i:i + block]
        index += 1

    return input_urls, url_dict


def get_proxy(proxy_index):
    return "{0}:{1}@{2}.torguardvpnaccess.com:6060".format(username, password, proxy_pnts[proxy_index])


def lookup_urls_all(clientId, urls, proxy_dict):
    global final_result
    sub_block = 10  # in practical use this value shoud be 500
    for i in range(0, len(urls), sub_block):
        res = lookup_urls(clientId, urls[i:i + sub_block], proxy_dict)
        if res is not None:
            for key in res:
                final_result[key] = res[key]


def lookup_urls(clientId, urls, proxy_dict):
    global result
    global api_key

    data = {
        "client": {
            "clientId": clientId,
            "clientVersion": "0.1"
        },
        "threatInfo": {
            "threatTypes":
                [
                    "MALWARE",
                    "SOCIAL_ENGINEERING",
                    "THREAT_TYPE_UNSPECIFIED",
                    "UNWANTED_SOFTWARE",
                    "POTENTIALLY_HARMFUL_APPLICATION"
                ],
            "platformTypes": ["ANY_PLATFORM"],
            "threatEntryTypes": ["URL"],
            "threatEntries": [{'url': u} for u in urls]
        }
    }
    headers = {'Content-type': 'application/json'}

    r = requests.post(
        'https://safebrowsing.googleapis.com/v4/threatMatches:find',
        data=json.dumps(data),
        params={'key': api_key},
        headers=headers,
        proxies=proxy_dict
    )
    if r.status_code == 200:
        # Return clean results
        if r.json() == {}:
            return dict([(u, {"malicious": False}) for u in urls])
        else:
            for url in urls:
                # Get matches
                matches = [match for match in r.json()['matches'] if match['threat']['url'] == url]
                if len(matches) > 0:
                    result[url] = {
                        'malicious': True,
                        'platforms': list(set([b['platformType'] for b in matches])),
                        'threats': list(set([b['threatType'] for b in matches])),
                        'cache': min([b["cacheDuration"] for b in matches])
                    }
                else:
                    result[url] = {"malicious": False}
            return result
    else:
        print("Some error has occurred while using google safe broswer POST api. Status code: {}".format(r.status_code))


if __name__ == '__main__':
    if len(sys.argv) == 4:
        inpf, username, password, api_keys = sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4]
        main()
    else:
        print("Usage: python gsb_process.py <input file>")



'''
Description: Get the aggregated VT record for a domain
Note: All time is taken as UTC time
'''
import sys
import psycopg2
import myutils
from datetime import datetime

def get_db_conn(db, user, passwd, host, port):
    conn = None
    try:
        conn = psycopg2.connect(database=db, user=user, password=passwd, host=host, port=port)
        return conn
    except Exception as e:
        print (e)
        pass
        return None

def get_record_(host, db, user, passwd, dbhost, port):
    conn = get_db_conn(db, user, passwd, dbhost, port)
    if conn == None:
        return None

    table = myutils.get_table(host)
    stmt = "SELECT firstseen, lastseen, count, mal_count, ts FROM " + table + " WHERE domain = %s"
    if table != None:
        params = (host,)
        try:
            cur = conn.cursor()
            cur.execute(stmt, params)
            return cur.fetchone()
        except Exception as e:
            print (e)
            pass
            return None
    else:
        return None

#returns an array (firstseen, lastseen, #queries, #mal_count, #timeseries)
def get_record(host, database):
    return get_record_(host, database, "pdns", "pdns", "10.4.8.61", 5432)

def format_record(mode, record):
    fr = ""
    if record != None:
        if mode == 'pretty':
            fr += "Domain: {}\n".format(sys.argv[1])
            fr += "Firstseen: {}\n".format(datetime.fromtimestamp(record[0]).strftime('%Y-%m-%d %H:%M:%S'))
            fr += "Lastseen: {}\n".format(datetime.fromtimestamp(record[1]).strftime('%Y-%m-%d %H:%M:%S'))
            fr += "Total count: {}\n".format(record[2])
            fr += "Mal count: {}\n".format(record[3])
            arr = record[4].split('##')
            ts = dict()
            for entry in arr:
                arr2 = entry.split()
                ts[arr2[0]] = arr2[1]
            count_str = ""
            for key, value in sorted(ts.items()):
                count_str += "{} ".format(value)
            fr += "Counts: {}".format(count_str)
        elif mode == 'csv':
            fr = "{},{},{},{},{},{}".format(sys.argv[1], record[0], record[1], record[2], record[3], record[4])
    return fr

if __name__ == "__main__":
    if len(sys.argv) < 3:
        print("Usage: {} <domain-name> <format [pretty|csv]> [database]".format(sys.argv[0]))
        exit()

    database = "vt_ts"
    if len(sys.argv) == 4:
        database = sys.argv[3].strip()

    record = get_record(sys.argv[1].strip(), database)
    print(format_record(sys.argv[2], record))



# Creating graphs and visualizing

#[g, edges, domains, ips] = create_graph("edges.csv")
#draw_domain_ip_graph(g, edges, domains, ips, mal_domains)

import networkx as nx
import matplotlib.pyplot as plt

def create_graph(edge_file):
    ef = open(edge_file)
    g = nx.Graph()
    edge_list = list()
    domains = set()
    ips = set()
    for line in ef:
        arr = line.strip().split()
        g.add_edge(arr[0], arr[1])
        domains.add(arr[0])
        ips.add(arr[1])
        edge_list.append((arr[0], arr[1]))
    return [g, edge_list, list(domains), list(ips)]

def draw_domain_ip_graph(g, edges, node_lists, labeling = 1, gw = 20, gh = 10, k = 0.15, itr = 20):
    plt.figure(figsize=(20,10))
    pos = nx.spring_layout(g,k=0.15,iterations=20)
    for node_list in node_lists:
        nodes = node_list[0]
        color = node_list[1]
        nx.draw_networkx_nodes(g, pos, nodelist = nodes, node_color = color)
    #nx.draw_networkx_nodes(g, pos, nodelist = domains, node_color = 'blue')
    #nx.draw_networkx_nodes(g, pos, nodelist = ips, node_color = 'green')
    #nx.draw_networkx_nodes(g, pos, nodelist = mal_domains, node_color = 'red')
    nx.draw_networkx_edges(g, pos, edge_list = edges, width = 2, edge_color = 'black')
    
    if labeling == 1:
        labels = dict()
        for node in g.nodes:
            labels[node] = node
        #for domain in domains:
        #    labels[domain] = domain
        #for ip in ips:
        #    labels[ip] = ip
        nx.draw_networkx_labels(g, pos, labels, font_size = 12)

def get_edge_set(all_edges, cc, outfile):
    edges = set()
    outf = open(outfile, "w")
    for edge in all_edges:
        if edge[0] == edge[1]:
            continue
        if edge[0] in cc:
            edges.add(edge)
    for edge in edges:
        outf.write("{} {}\n".format(edge[0], edge[1]))
    outf.close()
    return edges

def get_cc_sizes(g):
    return [len(c) for c in sorted(nx.connected_components(g), key=len, reverse=True)]

def get_cc_nodes(g):
    cc_nodes = []
    for cc in nx.connected_components(g):
        cc_nodes.append(g.subgraph(cc).copy().nodes)



import hashlib
import os
import re

import pandas as pd

from commons.utils.maxmind_tools import MyASN

#### global variables############################################################
# Load brands considered
# TODO: make global variables local and load them in main program
brands_df = pd.read_csv("../data/pt_alexa_brands_20210108.csv", header=None)
brands_df.columns = ["brand", "domain", "description"]
brands = set(brands_df["brand"].unique())

parking_df = pd.read_csv("../data/park_ns.txt", header=None, sep=" ")
parking_df.columns = ["name", "ns", "domain"]
parking = set(parking_df["domain"].unique())

asn = MyASN("/export/sec03/asn/GeoIPASN_each_day/geo_20210430_GeoIPASNum.dat")

####################################################################################
# utils
import pickle


# save data structure to a file
def save_ds(ds, filename):
    pickle.dump(ds, open(filename, 'wb'))


# load the data structure from file
def load_ds(filename):
    return pickle.load(open(filename, 'rb'))


def line_prepender(filename, line):
    if not os.path.exists(filename):
        open(filename, "w").close()

    with open(filename, 'r+') as f:
        content = f.read()
        f.seek(0, 0)
        f.write(line.rstrip('\r\n') + '\n' + content)


def read_first_line(filename):
    first_line = None
    if os.path.exists(filename):
        with open(filename) as f:
            first_line = f.readline().strip()
    return first_line


def get_sha256_hex(s):
    return hashlib.sha256(s.encode('utf-8')).hexdigest()


def contains_ip(st):
    if re.match('\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}', st) != None:
        return True
    return False



'''
Description: aggreegate VT hosts for a given time period
'''
from datetime import date, timedelta, datetime
import sys
import os
#v3
#from urllib.parse import urlparse
#v27
from urlparse import urlparse
import fnmatch
import time
import tldextract

BASE_VT_DIR = "/export/sec04/vt/vt_url_feed_parsed_v2"

def get_vt(startdate, enddate):
    os.environ["TZ"] = "UTC"
    s_date = datetime.strptime(startdate.strip(), "%Y%m%d").date()
    e_date = datetime.strptime(enddate.strip(), "%Y%m%d").date()
    delta = e_date - s_date
    vt = dict()
    hosts = set()
    for i in range(delta.days + 1):
        curr_date = s_date + timedelta(i)
        name = "res.url-feed-{}T??".format(curr_date.strftime("%Y%m%d"))
        filenames = fnmatch.filter(os.listdir(BASE_VT_DIR), name)
        for filename in filenames:
            full_path = os.path.join(BASE_VT_DIR, filename)
            fo = open(full_path)
            for line in fo:
                #http://www.gorillawalker.com/my-blue-is-happy.pdf##2019-05-31 23:00:18##9bccab2dde30fcb77d7c91c3c6b1dabe188f3e3f00645de2722ce951494b5cd1-1559343618##5##CyRadar:malicious site#Avira:phishing site#Forcepoint ThreatSeeker:phishing site#Sophos:malicious site#Fortinet:phishing site
                arr = line.strip().split('##')
                if len(arr) != 5:
                    print("ERROR: Invalid line: {}".format(line))
                    continue
                url = arr[0]
                if url == None or len(url) == 0:
                    print("ERROR: Invalid URL: {}".format(line))
                    continue

                datestr = arr[1]
                try:
                    #epoch = int(time.mktime(datetime.strptime(datestr, "%Y-%m-%d %H:%M:%S").date().timetuple()))
                    epoch = int(datetime.strptime(datestr, '%Y-%m-%d %H:%M:%S').strftime('%s'))
                except Exception as e:
                    print("ERRROR - likely format issue: {}".format(line))
                    print(e)
                    pass
                    continue
                #scheme='https', netloc='ww.xy.com', path='/', params='', query='', fragment=''
                try:
                    uri = urlparse(url)
                    host = uri.netloc
                    ext = tldextract.extract(host)
                    domain = ext.domain + "." + ext.suffix
                except Exception as e:
                    print("ERROR - something went wrong while parsing the URL: {}".format(url))
                    print(e)
                    pass
                    continue


                if domain == None or len(domain) == 0:
                    print("ERROR: Invalid Domain: {}".format(line))
                    continue
                scan_id = arr[2]
                if scan_id == None or len(scan_id) == 0:
                    print("ERROR: Invalid scan id: {}".format(line))
                    continue

                if domain not in vt:
                    vt[domain] = dict()
                    vt[domain]['hosts'] = set()
                    vt[domain]['firstseen'] = epoch
                    vt[domain]['lastseen'] = epoch
                    vt[domain]['total'] = 0
                    vt[domain]['mal_count'] = 0

                vt[domain]['total'] += 1
                if int(arr[3]) > 0:
                    vt[domain]['mal_count'] += 1

                if vt[domain]['firstseen'] > epoch:
                    vt[domain]['firstseen'] = epoch

                if vt[domain]['lastseen'] < epoch:
                    vt[domain]['lastseen'] = epoch

                #postgres can't handle very large text fields - so summarize long host names
                if len(host) > 100:
                    host_str = "l0ngh0st." + str(len(host)) + "." + str(host.count('.'))
                    vt[domain]['hosts'].add(host_str)
                else:
                    vt[domain]['hosts'].add(host)
    print("#Domains = {}".format(len(vt)))
    count = 0
    for key, value in vt.items():
        count += 1
        if count == 5:
            break
        print("{}:{}".format(key, value))
    return vt



from datetime import datetime
import os
from datetime import date, timedelta

os.environ["TZ"] = "UTC"

def get_dt(dstr, dformat = "%b %d %Y %I:%M %p"):
    dstr = dstr.replace("th", "").replace("st", "").replace("nd", "").replace("rd", "")
    return datetime.strptime(dstr, dformat)

def epoch2dt(epoch, dformat = "%Y-%m-%dT%H:%M:%S.%fZ"):
    return datetime.fromtimestamp(epoch).strftime(dformat) #'%Y-%m-%d %H:%M:%S')

def ts2dt(ts, dformat = "%Y-%m-%dT%H:%M:%S.%fZ"):
    return datetime.strptime(ts, dformat) #'%Y-%m-%d %H:%M:%S'

def dt2epoch(dt):
    return dt.timestamp()

#get duration in days
def get_duration(row):
    return (ts2dt(row["lastseen"]).timestamp() - ts2dt(row["firstseen"]).timestamp())/86400

def dt2str(dt, dformat = "%Y-%m-%dT%H:%M:%S.%fZ"):
    return dt.strftime(dformat)


##specific to VT
# handle multiple dateformats in VT historical whois and conver to epoch time
def vtdate2epoch(datestr):
    date_formats = ["%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%dT%H:%M:%S.%fZ", "%d-%b-%Y %H:%M:%S",
               "%Y-%m-%d %H:%M:%S", "%Y-%m-%d", "%d.%m.%Y %H:%M:%S", "%d/%m/%Y %H:%M:%S",
               "%Y-%m-%d %H:%M:%S%z", "%d-%b-%Y", "%Y-%m-%d %H:%M:%S.%f", "%Y%m%d",
               "%d.%m.%Y", "%B %d %Y", "%Y-%m-%dT%H:%M:%S.%f", "%a, %d %b %Y %H:%M:%S",
                "%Y.%m.%d %H:%M:%S" ]
    epoch = None
    
    if datestr != None and datestr != "" and datestr != float:
        ds = [datestr]
        if "|" in datestr:
            ds = [x.strip() for x in datestr.split("|")]
        for dformat in date_formats:
            found = False
            
            for d in ds:
                #exclude +02, +03  etc.
                try:
                    plus = d.index("+")
                    d = d[0:plus]
                except ValueError as e:
                    pass
                try:
                    utc = d.index("UTC")
                    d = d[0:utc].strip()
                except ValueError as e2:
                    pass

                try:
                    epoch = dt2epoch(ts2dt(d, dformat))
                    found = True
                    break
                except:
                    pass
            if found == True:
                break
        if found == False:
            print(datestr)
    return epoch



'''
Description: VT NOD
Author: @nabeelxy
Note:
- This represents queries made by users (may not necessarily new scans)
'''

import sys
import psycopg2
import time
import myutils
import agg_hosts
import agg_domains


def update_vt_url(vt, databasename):
    try:
        conn = psycopg2.connect(database=databasename, user="pdns", password="pdns", host = "10.4.8.61", port=5432)
        print("Connected")
    except:
        print ("I am unable to connect to the database")
        return

    cursors = dict()
    insert_cursors = dict()

    prefixes = myutils.get_prefixes_new()
    for prefix in prefixes:
        cursors[prefix] = conn.cursor()
        insert_cursors[prefix] = conn.cursor()

    #print cursors
    count = 0
    insert_count = 0
    update_count = 0
    duplicate_count = 0
    nulldomain = 0
    nullip = 0
    emptydomain = 0
    emptyip = 0
    start_time = time.time()
    for cid in cursors:
        cursors[cid].execute("BEGIN")

    for cid in insert_cursors:
        insert_cursors[cid].execute("BEGIN")

    blocksize = 100000
    for host, data in vt.items():
        if host == None or len(host) == 0:
            continue
        if count % blocksize == 0:
            s_time = time.time()
            for cid in cursors:
                cursors[cid].execute("COMMIT")
                conn.commit()
                cursors[cid].execute("BEGIN")
                e_time = time.time()
                elapsed_time = e_time - s_time
                #print ("line " + str(count) + " inserts = " + str(insert_count) + " updates = " + str(update_count) + " time = " + str(elapsed_time) + '\r')
        firstseen = data['firstseen']
        lastseen = data['lastseen']
        seen_count = data['total']
        mal_count = data['mal_count']
        ts = ""
        if databasename == "vt_url_ts":
            for scan_id, result in data['scans'].items():
                if len(ts) > 0:
                    ts += '##'
                ts += scan_id + '#' +result
        elif databasename == "vt_nod":
            for host2 in data['hosts']:
                if len(ts) > 0:
                    ts += '##'
                ts += host2            

        count += 1
            
        tablename = myutils.get_table(host)
        cursor_key = myutils.get_cursor_key(host)
        try:
            cursor = cursors[cursor_key]
        except KeyError:
            print("KeyError: save to special: Domain: {}".format(host))
            cursor = cursors["special"] #TODO: get rid of hard coded keys
            
        update_stmt = None
        params = None
        try:
            update_stmt = "UPDATE " + tablename + " SET firstseen = least(firstseen, %s), lastseen = greatest(lastseen, %s), count = count + %s, mal_count = mal_count + %s, ts = ts || '##' || %s WHERE domain = %s"; 
            params = (firstseen, lastseen, seen_count, mal_count, ts, host)
        except Exception as e:
            print("ERROR: Likely a type error in the update statement")
            print(e)
            pass
            continue

        try:
            cursor.execute(update_stmt, params)
        except ValueError:
            print("ValueError: update {}".format(host))

        if (cursor.rowcount == 0):
            insertquery = None
            params1 = None
            try:
                insertquery = "INSERT INTO " + tablename + " (domain, firstseen, lastseen, count, mal_count, ts) VALUES (%s, %s, %s, %s, %s, %s);" 
                params1 =  (host, firstseen, lastseen, seen_count, mal_count, ts)
            except Exception as e:
                print("ERROR: Likely a type error in the insert statement")
                print(e)
                pass
                continue
            try:
                insert_cursors[cursor_key].execute(insertquery, params1);
            except ValueError:
                print("ValueError: insert {}".format(host))
            insert_count += 1
        else:
            update_count += 1
    s_time = time.time()
    for cid in cursors:
        cursors[cid].execute("COMMIT")
        conn.commit()
        cursors[cid].close()
    e_time = time.time()
    elapsed_time = e_time - s_time
    print ("final update time = " + str(elapsed_time) + "\r")

    #commit inserts only at the end
    s_time = time.time()
    for cid in insert_cursors:
        insert_cursors[cid].execute("COMMIT")
        conn.commit()
        insert_cursors[cid].close()
    e_time = time.time()
    elapsed_time = e_time - s_time
    print ("final insert time = " + str(elapsed_time) + "\r")

    conn.close()

    if count != 0:
        print ("total {0} insert {1} update {2} duplicate {3}\n".format(count, insert_count, update_count, duplicate_count))
        print ("null domain {0} | empty domain {1} | null ip {2} | empty ip {3}\n".format(nulldomain, emptydomain, nullip, emptyip))
        print ("records use {0}: {1} seconds, average time per record {2} seconds.\n".format(count, time.time() - start_time, (time.time() - start_time) / count))
    else:
        print ("zero record inserted\n")

if __name__=="__main__":
    if len(sys.argv) < 3:
        print ("Usage:", sys.argv[0], "<startdate YYYYMMDD> <enddate YYYYMMDD> <databasename [vt_url_ts|vt_nod]>")
        sys.exit()
    startdate = sys.argv[1]
    enddate = sys.argv[2]
    database = sys.argv[3]

    vt = None
    if database == 'vt_url_ts':
        vt = agg_hosts.get_vt(startdate, enddate)
    elif database == 'vt_nod':
        vt = agg_domains.get_vt(startdate, enddate)
    else:
        print("ERROR: Invalid database")
        exit()

    update_vt_url(vt, database)



'''
Gather VT File data
'''
import vt_private as vtp
import sys
import json
from urllib.parse import urlparse
import time

SPEED = 0.1
def vt_file_file(hash_file, outfile, relation, keyindex):
    of = open(outfile, "w+")
    with open(hash_file) as df:
        for hashf in df:
            time.sleep(SPEED)
            hashf = hashf.strip()
            of.write("{}\n".format(json.dumps(vtp.getFile(hashf, relation, keyindex))))
    of.close()

if __name__ == "__main__":
    if len(sys.argv) < 5:
        print("Usage: {} <hash-file> <output-file> <relationship> <keyindex>".format(sys.argv[0]))
        exit()
    hashfile = sys.argv[1]
    outputfile = sys.argv[2]
    #contacted_domains
    relation = sys.argv[3]
    keyindex = int(sys.argv[4])

    vt_file_file(hashfile, outputfile, relation, keyindex)



import vt_query_host as vtq
import sys
import tldextract
import entropy
from datetime import datetime

def get_set(filename):
    words = set()
    fn = open(filename)
    for word in fn:
        words.add(word.strip())
    return words

def fs_ls(infilename, outfilename):
    infile = open(infilename)
    outfile = open(outfilename, "w+")

    for domain in infile:
        domain = domain.strip()
        record = vtq.get_record(domain, "vt_nod")
        if record == None:
            continue
        outfile.write("{}##{}##{}##{}##{}\n".format(domain, record[0], 
                     record[1], datetime.fromtimestamp(record[0]).strftime('%Y-%m-%d %H:%M:%S'),
                     datetime.fromtimestamp(record[1]).strftime('%Y-%m-%d %H:%M:%S'))) 
    outfile.close()

def subdomains(infilename, outfilename, threshold):
    infile = open(infilename)
    outfile = open(outfilename, "w+")

    sub_freq = dict()
    for domain in infile:
        domain = domain.strip()
        record = vtq.get_record(domain, "vt_nod")
        if record == None:
            continue
        fqdns = record[4].split('##')
        for fqdn in fqdns:
            ext = tldextract.extract(fqdn)
            if ext.subdomain == None or len(ext.subdomain) == 0:
                continue
            if '.' in ext.subdomain:
                continue
            if ext.subdomain not in sub_freq:
                sub_freq[ext.subdomain] = 0
            sub_freq[ext.subdomain] += 1
    print(sub_freq)
    for subdom, freq in sub_freq.items():
        if freq > threshold:
            outfile.write("{} {}\n".format(subdom, freq))
    outfile.close()

if __name__ == "__main__":
    method = sys.argv[1] #extract,subdoms
    if method == "extract":
        fs_ls(sys.argv[2], sys.argv[3])
    elif method == "subdoms":
        subdomains(sys.argv[2], sys.argv[3], int(sys.argv[4]))



import requests
import json
import sys

GSB_FETCH_URL = "https://safebrowsing.googleapis.com/v4/threatListUpdates:fetch"

#minhaj
#api_keys = ['AIzaSyCtXkzebCAfuISFqAog9pep_sSvY1-9SWY']

#digestize
api_keys = ['AIzaSyCXUW2uCTHoSyLjAIEKzNRPUIXCfG3MScU']


def fetch_db(outfile, key_index = 0):
    params = {'apikey': api_keys[key_index]}
    output = "{'Empty': 'Empty'}"
    try:
        req = requests.post(GSB_FETCH_URL, data=params)
        if req.status_code == 200:
            output = req.json()
            if output['response_code'] == -1:
                output['url'] = url
        elif req.status_code == 204:
            output = {'ERROR': 'Received HTTP 204: You may have exceeded the rate limit', 'url':url}
    except requests.ConnectTimeout as e:
        print(e)
        output = {'ERROR': '##timed out', 'url':url}

    of = open(outfile, "w")
    of.write("{}".format(json.dumps(output)))

fetch_db(sys.argv[1])



import matplotlib.pyplot as plt
import numpy as np

#data = [[df1, column1, low_limit1, high_limit1, label1, linestyle, color], ...]
def plot_numeric_cdf(data, x_label, xlim = None, ylim = None):
    for item in data:
        df = item[0]
        column_name = item[1]
        low_limit = item[2]
        high_limit = item[3]
        label = item[4]
        linestyle = item[5]
        color = item[6]
        if low_limit == None and high_limit == None:
            df_l = df
        elif low_limit == None:
            df_l = df[df[column_name] <= high_limit]
        elif high_limit == None:
            df_l = df[df[column_name] >= low_limit]
        else:
            df_l = df[(df[column_name] >= low_limit) & (df[column_name] <= high_limit)]
            
        df_g = df_l.groupby([column_name]).size().reset_index(name = "count")
        cdf = np.cumsum(df_g['count']/np.sum(df_g['count']))
        plt.plot(df_g[column_name], cdf, linestyle = linestyle, color = color, label = label)
    if ylim == None:
        plt.ylim((0,1))
    else:
        plt.ylim(ylim)
    
    if xlim != None:
        plt.xlim(xlim)
    plt.ylabel("CDF")
    plt.xlabel(x_label)
    plt.grid(True)
    plt.legend()
    plt.show()

def plot_pie(categories, values):
    fig = plt.figure()
    ax = fig.add_axes([0,0,1,1])
    ax.axis('equal')
    ax.pie(values, labels = categories,autopct='%1.2f%%')
    plt.show()



from commons.utils.date_tools import vtdate2epoch


# convert the first entry in VirusTotal WHOIS record (JSON object) into a feature dictionary
# TODO: consider historical WHOIS
def vtwhois2csv(vtj):
    features = dict()
    features['domain'] = None
    features['registrant'] = None
    features['registrar'] = None
    features['creation'] = None
    features['update'] = None
    features['expiration'] = None
    features['ns'] = None
    features['whoiserver'] = None

    try:
        whois_all = vtj["data"]
        if len(whois_all) == 0:
            return features

        # only consider the latest entry
        whois = whois_all[0]

        try:
            domain = whois["attributes"]["whois_map"]["Domain Name"].lower()
        except:
            pass
            # if there are more than 1 whois entry, try the second entry
            if len(whois_all) > 1:
                whois = whois_all[1]
                # if domain name is not available in the second attempt, we will not check further
                try:
                    domain = whois["attributes"]["whois_map"]["Domain Name"].lower()
                except:
                    pass

        features['domain'] = domain

        try:
            features['creation'] = vtdate2epoch(whois["attributes"]["whois_map"]["Creation Date"])
        except:
            pass
            # if there are more than 1 whois entry, try the second entry
            if len(whois_all) > 1:
                whois = whois_all[1]
                try:
                    features['creation'] = vtdate2epoch(whois["attributes"]["whois_map"]["Creation Date"])
                except:
                    pass

        try:
            features['registrant'] = whois["attributes"]["registrant_name"].lower()
        except:
            pass

        try:
            features['registrar'] = whois["attributes"]["whois_map"]["Registrar"].lower()
        except:
            pass
            try:
                features['registrar'] = whois["attributes"]["registrar_name"].lower()
            except:
                pass

        try:
            features['expiration'] = vtdate2epoch(whois["attributes"]["whois_map"]["Registry Expiry Date"])
        except:
            pass

        try:
            features['update'] = vtdate2epoch(whois["attributes"]["whois_map"]["Updated Date"])
        except:
            pass

        try:
            features['ns'] = whois["attributes"]["whois_map"]["Name Server"].lower()
        except:
            pass

        try:
            features['whoiserver'] = whois["attributes"]["whois_map"]["Registrar WHOIS Server"].lower()
        except:
            pass
    except:
        pass
    return features



import pdns
import sys
import os

def save_all(domain, folder, limit):
    entries = pdns.pdns_domain(domain, None, None, None, None, limit)
    filename = os.path.join(folder, domain)
    fh = open(filename, "w")
    if entries != None:
        for entry in entries:
            fh.write("{}\n".format(entry))
    fh.close()

def all_domains(domainf, folder, limit):
    df = open(domainf)
    for domain in df:
        domain = domain.strip().strip(".").encode("idna").decode()
        save_all(domain, folder, limit)

if __name__ == "__main__":
    domainfile = sys.argv[1]
    outfolder = sys.argv[2]
    limit = int(sys.argv[3])
    all_domains(domainfile, outfolder, limit)

    



import ip2cloud
import sys

i2c = ip2cloud.IP2Cloud("cloud_ips.txt")

ipf = open(sys.argv[1])
outf = open(sys.argv[2], "w+")
for ip in ipf:
    ip = ip.strip().strip("\r\n")
    cloud = i2c.get(ip)
    if cloud == None:
        cloud = "none"
    outf.write("{},{}\n".format(ip, cloud))
outf.close()




import tldextract
from urllib.parse import urlparse
import re

#check if a given text corresponds to an IP address
def is_ip(x):
    pat = re.compile("^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$")
    if pat.match(x):
        return True
    return False

def contains_ip(x):
    pat = re.compile(".*\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}.*")
    if pat.match(x):
        return True
    return False

def get_2ld(fqdn, part = "apex"):
    try:
        ext = tldextract.extract(fqdn)
        if len(ext.domain) == 0 or len(ext.suffix) == 0:
            return None
        else:
            if part == "tld":
                return ext.suffix
            elif part == "domain":
                return ext.domain
            else: 
                return ext.domain + "." + ext.suffix
    except Exception as e:
        #print("ERROR - something wrong with the FQDN: {}".format(fqdn))
        #print(e)
        pass
    return None

def get_fqdn(url):
    try:
        uri = urlparse(url)
        if uri != None:
            return uri.netloc
    except Exception as e:
        pass
    return None

def get_path(url):
    try:
        uri = urlparse(url)
        if uri != None:
            return uri.path
    except Exception as e:
        pass
    return None

def get_query(url):
    try:
        uri = urlparse(url)
        if uri != None:
            return uri.query
    except Exception as e:
        pass
    return None

def get_scheme(url):
    try:
        uri = urlparse(url)
        if uri != None:
            return uri.scheme
    except Exception as e:
        pass
    return None



#Compared to parse_vt - this adds the IP as well
import tarfile
import json
import glob
import sys
import os
import time
from urllib.parse import urlparse
from datetime import date, timedelta, datetime

BASE_VT_DIR = "/export/sec03/vt/vt_url_feed"
PARSED_VT_DIR = "/export/sec04/vt/vt_url_feed_parsed_v3"


def parse_vt_files(startdate, enddate):
    s_date = datetime.strptime(startdate.strip(), "%Y%m%d").date()
    e_date = datetime.strptime(enddate.strip(), "%Y%m%d").date()
    delta = e_date - s_date
    for i in range(delta.days + 1):
        curr_date = s_date + timedelta(i)
        files = glob.glob("{}/url-feed-{}T*.tar.bz2".format(BASE_VT_DIR, curr_date.strftime("%Y%m%d")))
        for fi in files:
            outf = "res.{0}".format(os.path.basename(fi).split('.')[0])
            extract_url_feed_tar(fi, os.path.join(PARSED_VT_DIR, outf))
            print("Parsed {}".format(os.path.basename(fi)))

def write_record(fd, record):
    try:
        url_obj = json.loads(record)
        url = url_obj["url"]
        scan_t = url_obj["scan_date"]
        scan_id = url_obj["scan_id"]
        positives = int(url_obj["positives"])
        ip = "NONE"
        try:
            ip = url_obj["additional_info"]["resolution"]
        except:
            pass
        avs = ""
        if positives > 0:
            scans = url_obj['scans']
            for av, result in scans.items():
                if result["detected"] == True:
                    if len(avs) > 0:
                        avs += '#'
                    avs += av + ':' + result['result']
        fd.write("{}##{}##{}##{}##{}##{}\n".format(url, scan_t, scan_id, positives, avs, ip))
    except Exception as e:
        print(e)
        print("ERROR: JSON object parsing error")  
     
def parse_vt_file(inf, outf):
    ih = open(inf)
    oh = open(outf, "w+")

    for line in ih:
        write_record(oh, line.strip())
    oh.close()

def extract_url_feed_tar(fi, outf, istar = True):
    fd = open(outf, "w+")
	
    try:
        tar = tarfile.open(fi, "r:bz2")
    except Exception as e:
        print(e)
        print("ERROR: parsing file {}".format(fi))
        return

    for item in tar:
        file_obj = tar.extractfile(item)
        if file_obj == None:
            continue

        for line in file_obj:
            write_record(fd, line.strip())
    tar.close()
    fd.close()

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print ("Usage: {} date <startdate> <enddate>".format(sys.argv[0]))
        print ("Usage: {} file <vtfile> <outfile>".format(sys.argv[0]))
        exit()
    option = sys.argv[1]
    if option == "date": 
        parse_vt_files(sys.argv[2], sys.argv[3])
    elif option == "file":
        parse_vt_file(sys.argv[2], sys.argv[3])



'''
Parse ADNS data collected into csv
'''
import sys
import datetime
from datetime import date, timedelta
import re

def isIp(tex):
    #if it is not match with regex it get error due to span
    try:
        return re.match('\d+\.\d+\.\d+\.\d+', tex).span()[1] == len(tex)
    except:
        return False

def get_date_obj(date_str):
    return datetime.datetime.strptime(date_str, "%Y%m%d").date()

def parser(values):
    node1=values[0]
    tableValue = [node1]

    if isIp(node1):
        print('First parameter is ip adress :' + node1)
        tableValue.append('ip')
    else:
        tableValue.append('domain')
        #domin name most of time contain '.' at the end of, remove it
        if node1.endswith('.'):
            node1 = node1[0:-1] #remove last charecter
            tableValue[0] = node1

    node2=values[-1]
    tableValue.append(node2)
    if isIp(node2):
        tableValue.append('ip')
    else:
        tableValue.append('domain')
        #domin name most of time contain '.' at the end of, remove it
        if node2.endswith('.'):
            node2 = node2[0:-1]  # remove last charecter
            tableValue[2] = node2

    #length of medium region does not known therefor first and last index except all value combined with ' '
    middleRegion = ''
    for str in values[1:-1]:
        middleRegion += str + ' '

    if 'IN A' in middleRegion:
        if 'domain' in tableValue[3]:
            print('ip type edge not domain2ip'+tableValue)
        tableValue.append('ip')
    elif 'MX' in middleRegion:
        tableValue.append('mx')
        #node2 contain mx score
        splitted = node2.split(':')
        node2=''
        for split in splitted[:-1]:
            node2 += split
        #last split removed
        if node2.endswith('.'):
            node2 = node2[0:-1]  # remove last charecter
        tableValue[2] = node2

    elif 'NS' in middleRegion:
        tableValue.append('ns')
    elif 'canonicalname' in middleRegion:
        tableValue.append('alias')
    # else:
    #     print('error unknown edge type')

    return tableValue

def parse_file(filename, outfile):
    outf = open(outfile, "w")
    with open(filename) as lines:
        for line in lines:
            values = line.split()
            if len(values) < 3:
                continue
            values = parser(values) #parse file line to database ordered data
            if len(values) != 5:
                continue

            if 'alias' in values[4]:
                #exclude cname records with same domains both sides
                if values[0] == values[2]:
                    continue
            outf.write("{}\n".format(" ".join(values)))

if __name__ == "__main__":
    if len(sys.argv) < 3:
        print("Usage: parser4adns.py <adns_file> <outfile>")
        sys.exit(0)
    input_file = sys.argv[1]
    output_file = sys.argv[2]
    parse_file(input_file, output_file)



"""
Get all phishtank URLs collected so far and do a full VT scan
"""
import glob
import pandas as pd
import os
import sys

VT_SRC_PATH = "/export/sec02/nabeel/COVID19-MAL/src/vt"

def chunks(l, n):
    for i in range(0, len(l), n):
        yield l[i:i+n]

def vt_all_urls(folder, datestr):
    all_urls = set()
    for filepath in glob.iglob('{}/fs_????????'.format(folder)):
        urls = set()
        with open(filepath, encoding = 'utf-8') as fh:
            for line in fh:
                urls.add(line.strip())
        all_urls = all_urls | urls

    splits = list(chunks(list(all_urls), 9000))
    count = 0
    for split in splits:
        filename = "{}/all_{}.{}".format(folder, datestr, count)
        with open(filename, 'w', encoding = 'utf-8') as fh:
            for item in split:
                fh.write("{}\n".format(item))
        count += 1
    
    #currently we only have 5 academic keys to process
    #this dirty hack is added to limit
    if count > 5:
        count = 5

    for i in range(0,count):
        #k = 4 - i
        #k = i
        filename = "{}/all_{}.{}".format(folder, datestr, i)
        os.system("nohup python {}/vt_file.py {} {}.scan {} &".format(VT_SRC_PATH, filename, filename, k))

if __name__ == "__main__":
    vt_all_urls(sys.argv[1], sys.argv[2])



import vt_query_host as vtq
import sys
def get_set(filename):
    words = set()
    fn = open(filename)
    for word in fn:
        words.add(word.strip())
    return words

def ext_features(infilename, outfilename):
    infile = open(infilename)
    outfile = open(outfilename, "w+")

    for domain in infile:
        domain = domain.strip()
        record = vtq.get_record(domain, "vt_ts")
        if record == None:
            continue
        lifetime = record[1] - record[0]
        total_count = record[2]    
        mal_count = record[3]
        ratio = float(mal_count)/total_count
        outfile.write("{} {} {} {} {}\n".format(domain, lifetime, total_count, mal_count, ratio))
    outfile.close()

if __name__ == "__main__":
    ext_features(sys.argv[1], sys.argv[2])



'''
Parse RDAP output to CSV format
'''
import sys

def parse_domain_whois(inputf, outputf, fields):
    inf = open(inputf)
    outf = open(outputf, "w")
    for line in inf:
        line = line.strip()

    outf.close()


def parse_ip_whois(inputf, outputf, fields):
    inf = open(inputf)
    outf = open(outputf, "w")
    


def parse_as(inputf, outputf, fields):
    inf = open(inputf)
    outf = open(outputf, "w")





'''
Extract lexical features
'''
import pandas as pd
import tldextract
from urllib.parse import urlparse
import re
import entropy
import math
from Levenshtein import distance
import sys
from brandseg import BrandSeg
from confusables import unconfuse
from suspicious import tlds, popular_keywords


#################################################################################
brands_df = pd.read_csv("pt_alexa_brands_20210108.csv", header = None)
brands_df.columns = ["brand", "domain", "description"]
brands = set(brands_df["brand"].unique())

#################################################################################

#check if a given text corresponds to an IP address
def is_ip(x):
    pat = re.compile("^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$")
    if pat.match(x):
        return True
    return False

def get_2ld(fqdn, part = "apex"):
    try:
        ext = tldextract.extract(fqdn)
        if len(ext.domain) == 0 or len(ext.suffix) == 0:
            return None
        else:
            if part == "tld":
                return ext.suffix
            elif part == "domain":
                return ext.domain
            else:
                return ext.domain + "." + ext.suffix
    except Exception as e:
        #print("ERROR - something wrong with the FQDN: {}".format(fqdn))
        #print(e)
        pass
    return None

#lexical features
def get_lex_features(domain, bseg, brands):
    features = dict()

    #segment the brand
    res = bseg.segment_domain(domain)
    sub_words = res[0]
    dom_words = res[1]
    all_words = sub_words + dom_words
    tld = res[2]

    
    features["domain"] = domain
    # Suspicious TLD
    features["suspicious_tld"] = 0
    for t in tlds:
        if t == tld:
            features["suspicious_tld"] = 1
            break

    # Remove initial '*.' for wildcard certificates bug
    if domain.startswith('*.'):
        domain = domain[2:]

    features["length"] = len(domain)

    # Entropy
    # Higher entropy is kind of suspicious
    features["entropy"] = entropy.shannon_entropy(domain)

    # IDN characters
    domain = unconfuse(domain)

    # Contains embedded TLD/ FAKE TLD
    features["fake_tld"] = 0
    #exclude tld
    for word in all_words:
        if word in ['com', 'net', 'org', 'edu', 'mil', 'gov']:
            features["fake_tld"] += 1

    ## No. of popular brand names appearing in domain name
    #features["brand"] = 0
    #for br in brands:
    #    for word in all_words:
    #        if br in word:
    #            features["brand"] += 1
    
    features["brand_pos"] = -1
    for brand in brands:
        i = domain.find(brand)
        if i >= 0:
            features["brand_pos"] = i
            break

    # Appearance of popular keywords
    features["pop_keywords"] = 0
    for word in popular_keywords:
        if word in all_words:
            features["pop_keywords"] += 1

    # Testing Levenshtein distance for keywords
    # Let's go for Levenshtein distance less than 2
    #features["similar"] = 0
    #for br in brands:
    #    # Removing too generic keywords (ie. mail.domain.com)
    #    for word in [w for w in all_words if w not in ['email', 'mail', 'cloud']]:
    #        if distance(str(word), str(br)) <= 2:
    #            features["similar"] += 1

    # Lots of '-' (ie. www.paypal-datacenter.com-acccount-alert.com)
    features["is_idn"] = 0
    if 'xn--' in domain:
        features["is_idn"] = 1
        features["minus"] = domain.count('-') - 2
    else:
        features["minus"] = domain.count('-')

    # Deeply nested subdomains (ie. www.paypal.com.security.accountupdate.gq)
    #features["num_subdomains"] = domain.count('.') - 1
    
    return features

#a simple test
if __name__ == "__main__":
    print(get_lex_features("www.paypal-datacenter.com-acccount-alert.com", BrandSeg(), brands))



import tarfile
import json
import glob
import sys
import os
import time
from urllib.parse import urlparse
from datetime import date, timedelta, datetime

BASE_VT_DIR = "/export/sec03/vt/vt_url_feed"
PARSED_VT_DIR = "/export/sec04/vt/vt_url_feed_parsed_v2"


def parse_vt_files(startdate, enddate):
    s_date = datetime.strptime(startdate.strip(), "%Y%m%d").date()
    e_date = datetime.strptime(enddate.strip(), "%Y%m%d").date()
    delta = e_date - s_date
    for i in range(delta.days + 1):
        curr_date = s_date + timedelta(i)
        files = glob.glob("{}/url-feed-{}T*.tar.bz2".format(BASE_VT_DIR, curr_date.strftime("%Y%m%d")))
        for fi in files:
            outf = "res.{0}".format(os.path.basename(fi).split('.')[0])
            extract_url_feed_tar(fi, os.path.join(PARSED_VT_DIR, outf))
            print("Parsed {}".format(os.path.basename(fi)))

def extract_url_feed_tar(fi, outf):
    fd = open(outf, "w+")
	
    try:
        tar = tarfile.open(fi, "r:bz2")
    except Exception as e:
        print(e)
        print("ERROR: parsing file {}".format(fi))
        return

    for item in tar:
        file_obj = tar.extractfile(item)
        if file_obj == None:
            continue

        for line in file_obj:
            try:
                url_obj = json.loads(line)
                url = url_obj["url"]
                scan_t = url_obj["scan_date"]
                scan_id = url_obj["scan_id"]
                positives = int(url_obj["positives"])
                avs = ""
                if positives > 0:
                    scans = url_obj['scans']
                    for av, result in scans.items():
                        if result["detected"] == True:
                            if len(avs) > 0:
                                avs += '#'
                            avs += av + ':' + result['result']
                fd.write("{}##{}##{}##{}##{}\n".format(url, scan_t, scan_id, positives, avs))
            except:
                print("ERROR: JSON object parsing error")       
                continue
    tar.close()
    fd.close()

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print ("Usage: {} <startdate> <enddate>".format(sys.argv[0]))
        exit()
    parse_vt_files(sys.argv[1], sys.argv[2])



#!/usr/bin/python
'''
Author: @nabeelxy
Description:
This file is specifically prepared for phicious project
Given a list of domains in a file, it produces a file with the following features:
'''

import sys
import pdns
from datetime import datetime
import tldextract
import csv
import pandas as pd

def get_2ld(fqdn):
    ext = tldextract.extract(fqdn)
    return ext.domain + "." + ext.suffix

#domain - domain name
#firstseen_before - None or any specific date
#num_ips - number of latest IPs to collect (some domains have thousands of IPs. So, limiting the collection
#useful. Old IPs anyway do not contribute much.
def get_pdns_features(domain, firstseen_before = None, num_ips = None):
    first_seen = None
    last_seen = None
    count = 0
    ip_set = set()
    first_seen_date = None
    last_seen_date = None
    LAST_SEEN_AFTER = "20150101 00:00"
    mappings = pdns.pdns_domain(domain, firstseen_before, None, None, LAST_SEEN_AFTER, 5000)
    mappings_ns = pdns.pdns_domain(get_2ld(domain), firstseen_before, None, None, LAST_SEEN_AFTER, 100, "NS")
    mappings_soa = pdns.pdns_domain(get_2ld(domain), firstseen_before, None, None, LAST_SEEN_AFTER, 100, "SOA")
    ns_set = set()
    soa_set = set()
    ns_domain = None #idicates if the ns domain is same as the apex domain under consideration
    soa_domain = None
    current_ip = None
    latest_ips = list()

    if mappings == None:
        return None
    
    if mappings_ns != None:
        for entry_ns in mappings_ns:
            for domain in entry_ns["rdata"]:
                domain = domain.strip(".") #remove ending dot, if any
                ns_set.add(get_2ld(domain))
        if get_2ld(domain) in ns_set:
            ns_domain = True

    if mappings_soa != None:
        for entry_soa in mappings_soa:
            for record in entry_soa["rdata"]:
                items = record.split()
                if len(items) > 2:
                    admin_domain = items[1].strip(".") #remove ending dot, if any
                    soa_set.add(get_2ld(admin_domain))
        if get_2ld(domain) in soa_set:
            soa_domain = True

    for entry in mappings:
        if first_seen == None:
            first_seen = entry["time_first"]
        elif first_seen > entry["time_first"]:
            first_seen = entry["time_first"]
        if last_seen == None:
            last_seen = entry["time_last"]
        elif last_seen < entry["time_last"]:
            last_seen = entry["time_last"]
        for ip in entry["rdata"]:
            if current_ip == None:
                current_ip = ip
            ip_set.add(ip)
            #maintain the ordered list (if number of ips to collect is specified)
            if num_ips != None and ip not in latest_ips and len(latest_ips) < num_ips:
                latest_ips.append(ip)
        count += entry["count"]

    #firstseen, lastseen, query count, #name servers, ns domain, #soa domains, soa_domain, current ip, all ips, all NSes, latest IPs
    return [first_seen, last_seen, count, len(ip_set), len(ns_set), ns_domain, len(soa_set), soa_domain, ip, ip_set, ns_set, latest_ips]

def get_features_file(domain_file, outfile, firstseen_before = None, num_ips = 20):
    df = open(domain_file, encoding = 'utf-8')
    of = open(outfile, "w+", encoding = 'utf-8')
    for domain in df:
        domain = domain.encode("idna").decode().strip()
        out = get_pdns_features(domain, firstseen_before, num_ips)
        duration = -1
        if out != None:
            first_seen_date = None
            last_seen_date = None
            if out[0] != None:
                first_seen_date = datetime.fromtimestamp(out[0])
            if out[1] != None:
                last_seen_date = datetime.fromtimestamp(out[1])
            if out[0] != None and out[1] != None:
                duration = (out[1] - out[0])/86400 # number of days
            of.write("{},{},{},{},{},{},{},{},{},{},{},\"{}\",\"{}\"\n".format(domain, first_seen_date, last_seen_date, out[2], out[3], out[4], out[5], out[6], out[7],
                                                              duration, out[8], out[10], out[11]))
    of.close()  
    df.close() 

def pdnscsv2df(csvfile):
    records = []
    with open(csvfile) as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=',')
        for row in csv_reader:
            records.append(row)
    
    df_pdns = pd.DataFrame().from_records(records)
    df_pdns.columns = ["domain",
                   "firstseen", 
                   "lastseen", 
                   "query count", 
                   "#ips",
                   "#name servers", 
                   "ns domain", 
                   "#soa domains", 
                   "soa_domain", 
                   "duration", 
                   "current ip", 
                   "all NSes", 
                   "latest IPs"]
    return df_pdns

if __name__ == "__main__":
    if len(sys.argv) < 4:
        print("usage: python {} input_domain_file output_file firstseen_before(YYYYMMDD HH:MM)".format(sys.argv[0]))
        exit()
        
    domainf = sys.argv[1] #input domain  file
    outf = sys.argv[2] #output file
    firstseen_before = sys.argv[3]
    if firstseen_before.lower() == "none":
        firstseen_before = None
    get_features_file(domainf, outf, firstseen_before)



'''
Description: VT TS - create a summary of VT records. Only consider FQDNs (not URLs)
Author: @nabeelxy
Note:
- This represents queries made by users (may not necessarily new scans)
'''

import sys
import os
import psycopg2
import time
import myutils
from datetime import timedelta, datetime
import fnmatch

BASE_VT_DIR = "/export/sec03/vt/vt_url_feed_parsed"


def append_files(startdate, enddate):                        
    s_date = datetime.strptime(startdate.strip(), "%Y%m%d").date()
    e_date = datetime.strptime(enddate.strip(), "%Y%m%d").date() 
    delta = e_date - s_date                                                       
    for i in range(delta.days + 1):                                               
        curr_date = s_date + timedelta(i)                                           
        name = "res.url-feed-{}T*.domain".format(curr_date.strftime("%Y%m%d"))
        filenames = fnmatch.filter(os.listdir(BASE_VT_DIR), name)
        for filename in filenames:
            full_path = os.path.join(BASE_VT_DIR, filename)
            print("Inserting {}".format(filename))
            update(full_path, "vt_ts")

def update(filename, databasename):
    try:
        conn = psycopg2.connect(database=databasename, user="pdns", password="pdns", host = "127.0.0.1", port=5432)
        print("Connected")
    except:
        print ("I am unable to connect to the database")
        return

    cursors = dict()
    insert_cursors = dict()

    prefixes = myutils.get_prefixes_new()
    for prefix in prefixes:
        cursors[prefix] = conn.cursor()
        insert_cursors[prefix] = conn.cursor()

    #print cursors
    count = 0
    insert_count = 0
    update_count = 0
    duplicate_count = 0
    nulldomain = 0
    nullip = 0
    emptydomain = 0
    emptyip = 0
    start_time = time.time()
    for cid in cursors:
        cursors[cid].execute("BEGIN")

    for cid in insert_cursors:
        insert_cursors[cid].execute("BEGIN")

    blocksize = 100000
    fn = open(filename)
    for line in fn:
        #boosterads.online##N##2019-08-04 16:58:49##0 0##2019-08-04 16:58:49
        values = line.split("##")
        if (len(values) != 5):
            print ("Invalid format {0}\n".format(line))
            continue
        domain = values[0]

        if domain == None or len(domain) == 0:
            print("WARNING: empty domain: {}".format(line))
            continue

        is_mal = values[1]
        datestr = values[2]
        epoch = None

        #conservatively exclude format errors
        try:
            epoch = int(time.mktime(datetime.strptime(datestr, "%Y-%m-%d %H:%M:%S").date().timetuple()))
        except Exception as e:
            print("ERRROR - likely format issue: {}".format(line))
            print(e)
            pass
            continue

        ts = "{} {}".format(epoch, values[3])

        if count % blocksize == 0:
            s_time = time.time()
            for cid in cursors:
                cursors[cid].execute("COMMIT")
                conn.commit()
                cursors[cid].execute("BEGIN")
            e_time = time.time()
            elapsed_time = e_time - s_time
            print ("line " + str(count) + " inserts = " + str(insert_count) + " updates = " + str(update_count) + " time = " + str(elapsed_time) + '\r')
        firstseen = epoch
        lastseen = epoch
        seen_count = 1
        count += 1
        mal_count = 0
        if is_mal == 'Y':
            mal_count = 1
            
        tablename = myutils.get_table(domain)
        cursor_key = myutils.get_cursor_key(domain)
        try:
            cursor = cursors[cursor_key]
        except KeyError:
            print("KeyError: save to special: Domain: {}".format(domain))
            cursor = cursors["special"] #TODO: get rid of hard coded keys

        update_stmt = None
        params = None
        try:
            update_stmt = "UPDATE " + tablename + " SET firstseen = least(firstseen, %s), lastseen = greatest(lastseen, %s), count = count + %s, mal_count = mal_count + %s, ts = ts || '##' || %s WHERE domain = %s"; 
            params = (firstseen, lastseen, seen_count, mal_count, ts, domain)
        except Exception as e:
            print("ERROR: Likely a type error in the update statement")
            print(e)
            pass
            continue

        try:
            cursor.execute(update_stmt, params)
        except ValueError:
            print("ValueError: update {}".format(domain))
        if (cursor.rowcount == 0):
            insertquery = None
            params1 = None
            try:
                insertquery = "INSERT INTO " + tablename + " (domain, firstseen, lastseen, count, mal_count, ts) VALUES (%s, %s, %s, %s, %s, %s);" 
                params1 =  (domain, firstseen, lastseen, seen_count, mal_count, ts)
            except Exception as e:
                print("ERROR: Likely a type error in the insert statement")
                print(e)
                pass
                continue
            try:
                insert_cursors[cursor_key].execute(insertquery, params1);
            except ValueError:
                print("ValueError: insert {}".format(domain))
            insert_count += 1
        else:
            update_count += 1
    s_time = time.time()
    for cid in cursors:
        cursors[cid].execute("COMMIT")
        conn.commit()
        cursors[cid].close()
    e_time = time.time()
    elapsed_time = e_time - s_time
    print ("final update time = " + str(elapsed_time) + "\r")

    #commit inserts only at the end
    s_time = time.time()
    for cid in insert_cursors:
        insert_cursors[cid].execute("COMMIT")
        conn.commit()
        insert_cursors[cid].close()
    e_time = time.time()
    elapsed_time = e_time - s_time
    print ("final insert time = " + str(elapsed_time) + "\r")

    conn.close()

    if count != 0:
        print ("total {0} insert {1} update {2} duplicate {3}\n".format(count, insert_count, update_count, duplicate_count))
        print ("null domain {0} | empty domain {1} | null ip {2} | empty ip {3}\n".format(nulldomain, emptydomain, nullip, emptyip))
        print ("records use {0}: {1} seconds, average time per record {2} seconds.\n".format(count, time.time() - start_time, (time.time() - start_time) / count))
    else:
        print ("zero record inserted\n")

if __name__=="__main__":
    if len(sys.argv) < 2:
        print ("Usage:", sys.argv[0], "<startdate YYYYMMDD> <enddate YYYYMMDD>")
        sys.exit()
    startdate = sys.argv[1]
    enddate = sys.argv[2]
    append_files(startdate, enddate)



'''
Description: DNSDB API wrapper
Author: @nabeelxy
'''

import os, json
import urllib
import sys
import contextlib

if sys.version_info[0] < 3:
    from urllib2 import Request, urlopen, HTTPError
else:
    from urllib.request import urlopen
    from urllib.error import HTTPError
from datetime import datetime

FARSIGHT_LOCAL_SERVER = "10.4.8.82"
FARSIGHT_API_KEY = "82827cf79576078112b80646585a583b081edcaba91bf6f7898c6ee1725febf7"

# Author: @nabeelxy
# Description:
# Farsight DNS access utility functions
# Farsight query parameters
# time_first_before - show records before the first seen datetime [UTC]
# time_first_after - show records after the first seen datetime [UTC]
# time_last_before - show records before the last seen datetime [UTC]
# time_last_after - show records after the last seen datetime [UTC]
# limit - limit the number of responses
# Reference: https://api.dnsdb.info/

# Build the get query parameters for query request to the farsight server
def build_query_params(first_before = None, first_after = None, last_before = None, last_after = None, limit = None):
  os.environ["TZ"] = "UTC"
  query_str = ""
  if (first_before != None):
    #TODO: change to UTC epoch time
    epoch_first_before = datetime.strptime(first_before, '%Y%m%d %H:%M').strftime('%s')
    if len(query_str) > 0:
      query_str += "&"
    query_str += "time_first_before="
    query_str += epoch_first_before

  if (first_after != None):
    epoch_first_after = datetime.strptime(first_after, '%Y%m%d %H:%M').strftime('%s')
    if len(query_str) > 0:
      query_str += "&"
    query_str += "time_first_after="
    query_str += epoch_first_after

  if (last_before != None):
    epoch_last_before = datetime.strptime(last_before, '%Y%m%d %H:%M').strftime('%s')
    if len(query_str) > 0:
      query_str += "&"
    query_str += "time_last_before="
    query_str += epoch_last_before

  if (last_after != None):
    epoch_last_after = datetime.strptime(last_after, '%Y%m%d %H:%M').strftime('%s')
    if len(query_str) > 0:
      query_str += "&"
    query_str += "time_last_after="
    query_str += epoch_last_after

  if (limit != None):
    if len(query_str) > 0:
      query_str += "&"
    query_str += "limit="
    query_str += str(limit)

  return query_str

# Get resolved IP for a given domain (only A records)
# Update: 20190924 - updated to return different types of resource records - A, AAA, SOA, MX, NS
# Dates in YYYYMMDD HH:MM format. Note that no validation is performed. 
# Caller must validate the datetime inputs before
# invoking this function
def pdns_domain(domain, first_before = None, first_after = None, last_before = None, last_after = None, limit = None, 
                rtype = "A", server = FARSIGHT_LOCAL_SERVER):
  query_str = build_query_params(first_before, first_after, last_before, last_after, limit)
  if len(query_str) > 0:
    query_str = "?" + query_str
    
  request = urllib.request.Request("http://{}/lookup/rrset/name/{}/{}{}".format(server, domain, rtype, query_str), 
                    headers={
                      "Accept":'application/json',
                      "X-API-Key": FARSIGHT_API_KEY
                    })
  ip_objects = None
  try:
    with contextlib.closing(urlopen(request)) as response:
        #response = urlopen(request)
        ip_objects = [json.loads(obj) for obj in response]
  except HTTPError:
    #handle 404's
    pass
  return ip_objects
    
# Get the domains that a given IP resolves to
def pdns_ip(ip, first_before = None, first_after = None, last_before = None, last_after = None, 
            limit = None, server = FARSIGHT_LOCAL_SERVER):
  query_str = build_query_params(first_before, first_after, last_before, last_after, limit)
  if len(query_str) > 0:
    query_str = "?" + query_str

  request = urllib.request.Request("http://{}/lookup/rdata/ip/{}{}".format(server, ip, query_str), 
                    headers={
                      "Accept":'application/json',
                      "X-API-Key": FARSIGHT_API_KEY
                    })
  ip_objects = None
  try:
    #response = urlopen(request)
    with contextlib.closing(urlopen(request)) as response:
        ip_objects = [json.loads(obj) for obj in response]
  except HTTPError:
    pass
  return ip_objects

def get_first_seen_date(mappings):
  first_seen = None
  first_seen_date = None
  if mappings == None:
   return None
  for entry in mappings:
    if first_seen == None:
      first_seen = entry["time_first"]
    elif first_seen > entry["time_first"]:
      first_seen = entry["time_first"]

  if first_seen != None:
    first_seen_date = datetime.utcfromtimestamp(first_seen)
  return first_seen_date

def get_first_last_seen_date(mappings):
  first_seen = None
  last_seen = None
  if mappings == None:
   return None
  for entry in mappings:
    if first_seen == None:
      first_seen = entry["time_first"]
    elif first_seen > entry["time_first"]:
      first_seen = entry["time_first"]
    if last_seen == None:
      last_seen = entry["time_last"]
    elif last_seen < entry["time_last"]:
      last_seen = entry["time_last"]
  return [first_seen, last_seen]

def pdns_ns(twold, first_before = None, first_after = None, last_before = None, last_after = None, 
            limit = None, server = FARSIGHT_LOCAL_SERVER):
  query_str = build_query_params(first_before, first_after, last_before, last_after, limit)
  if len(query_str) > 0:
    query_str = "?" + query_str
  #print("http://{}/lookup/rdata/{}/ns{}".format(FARSIGHT_LOCAL_SERVER, twold, query_str))
  request = urllib.request.Request("http://{}/lookup/rrset/name/{}/ns{}".format(server, twold, query_str),
                    headers={
                      "Accept":'application/json',
                      "X-API-Key": FARSIGHT_API_KEY
                    })
  pdns_objects = None
  try:
    #response = urlopen(request)
    with contextlib.closing(urlopen(request)) as response:
        pdns_objects = [json.loads(obj) for obj in response]
  except HTTPError:
    pass
  return pdns_objects




import requests

VT_SCAN_URL = 'https://www.virustotal.com/vtapi/v2/url/scan'
VT_REPORT_URL = 'https://www.virustotal.com/vtapi/v2/url/report'
VT_DOMAIN = 'https://www.virustotal.com/api/v3/domains/{}/{}'
VT_IP = 'https://www.virustotal.com/api/v3/ip_addresses/{}/{}'
VT_FILE = 'https://www.virustotal.com/api/v3/files/{}/{}'


def scan(url, api_keys, key_index=0):
    params = {'apikey': api_keys[key_index], 'url': url}
    output = {}
    try:
        req = requests.post(VT_SCAN_URL, data=params)
        if req.status_code == 200:
            output = req.json()
            if output['response_code'] == -1:
                output['url'] = url
        elif req.status_code == 204:
            output = {'ERROR': 'Received HTTP 204: You may have exceeded the rate limit', 'url': url}
    except requests.ConnectTimeout as e:
        output = {'ERROR': '##timed out', 'url': url}
    return output


def getReport(resource, api_keys, key_index=0):
    params = {'apikey': api_keys[key_index], 'resource': resource, 'allinfo': 'true'}
    output = {}
    try:
        req = requests.get(VT_REPORT_URL, params=params)
        if req.status_code == 200:
            output = req.json()
        elif req.status_code == 204:
            output = {'ERROR': 'Received HTTP 204: You may have exceeded the rate limit', 'url': resource}
    except requests.ConnectTimeout as e:
        output = {'ERROR': '##timed out', 'url': resource}
    return output


def getDomain(domain, relationship, api_keys, key_index=0, limit=None):
    querystring = None
    url = VT_DOMAIN.format(domain, relationship)
    output = {}
    if limit != None:
        querystring = {"limit": "{}".format(limit)}
    try:
        headers = {'x-apikey': api_keys[key_index]}
        response = requests.request("GET", url, headers=headers, params=querystring)
        if response.status_code == 200:
            output = response.json()
        else:
            output = {'ERROR': 'Received HTTP Code <> 200'}
    except requests.ConnectTimeout as e:
        output = {'ERROR': '##timed out', 'url': url}
    return output


def getIP(ip, relationship, api_keys, key_index=0, limit=None):
    querystring = None
    url = VT_IP.format(ip, relationship)
    output = {}
    if limit != None:
        querystring = {"limit": "{}".format(limit)}
    try:
        headers = {'x-apikey': api_keys[key_index]}
        response = requests.request("GET", url, headers=headers, params=querystring)
        if response.status_code == 200:
            output = response.json()
        else:
            output = {'ERROR': 'Received HTTP Code <> 200'}
    except requests.ConnectTimeout as e:
        output = {'ERROR': '##timed out', 'url': url}
    return output


def getFile(hashf, relationship, api_keys, key_index=0, limit=None):
    querystring = None
    url = VT_FILE.format(hashf, relationship)
    output = {}
    if limit != None:
        querystring = {"limit": "{}".format(limit)}
    try:
        headers = {'x-apikey': api_keys[key_index]}
        response = requests.request("GET", url, headers=headers, params=querystring)
        if response.status_code == 200:
            output = response.json()
        else:
            output = {'ERROR': 'Received HTTP Code <> 200: {}'.format(response.status_code)}
    except requests.ConnectTimeout as e:
        output = {'ERROR': '##timed out', 'url': url}
    return output



'''
Collect VT Host related data
'''
import vt_private as vtp
import sys
import json
from urllib.parse import urlparse
import time
import tldextract 

SPEED = 0.1
def get_2ld(fqdn, only_tld = False):
    try:
        ext = tldextract.extract(fqdn)
        if len(ext.domain) == 0 or len(ext.suffix) == 0:
            return None
        else:
            if only_tld:
                return ext.suffix
            else:
                return ext.domain + "." + ext.suffix
    except Exception as e:
        print("ERROR - something wrong with the FQDN: {}".format(fqdn))
        print(e)
        pass
    return None

def dcert_file(domain_file, outfile, relation, keyindex):
    of = open(outfile, "w+")
    with open(domain_file) as df:
        for line in df:
            time.sleep(SPEED)
            of.write("{}\n".format(json.dumps(vtp.getDomain(line.strip(), relation, keyindex))))
    of.close()

if __name__ == "__main__":
    if len(sys.argv) < 5:
        print("Usage: {} <domain-file> <output-file> <relationship> <keyindex>".format(sys.argv[0]))
        exit()

    domainfile = sys.argv[1]
    outputfile = sys.argv[2]
    #resolutions,siblings,subdomains,referrer_files 
    relation = sys.argv[3]
    keyindex = int(sys.argv[4])
    dcert_file(domainfile, outputfile, relation, keyindex)



from wordsegment import load, segment
import tldextract
import re

class BrandSeg:
    def __init__(self):
        load()

    def segment_domain(self, domain):
        is_wild_card = False
        if domain.startswith('*'):
            is_wild_card = True
        ext = tldextract.extract(domain)
        subdomain_list = list()
        tld = ext.suffix
        if len(ext.subdomain) > 0:
            subdomain_list = self.segment_part(ext.subdomain)
        apex_list = self.segment_part(ext.domain)
        return [subdomain_list, apex_list, tld, is_wild_card]

    def segment_part(self, part):
        words = re.split("\W+", part)
        words_list = list()
        for word in words:
            words_list.append(segment(word))
        final_list = list()
        for value in words_list:
            for word in value:
                final_list.append(word)
        return final_list



brands = {
#large corporations
    'apple',
    'google',
    'microsoft',
#social media
    'facebook',
    'twitter',
    'tumblr',
    'reddit',
    'youtube',
    'linkedin',
    'instagram',
    'flickr',
    'whatsapp',
#banking
    'paypal',
    'bankofamerica',
    'moneygram',
    'westernunion',
    'wellsfargo',
    'citigroup',
    'santander',
    'morganstanley',
    'barclays',
    'hsbc',
    'scottrade',
    'ameritrade',
    'merilledge',
    'chase',
#email
    'gmail',
    'hotmal',
    'protonmail',
    'yahoo',
    'outlook',
    'office365',
#e-commerce
    'amazon',
    'overstock',
    'alibaba',
    'aliexpress',
    'leboncoin',
    'ebay',
    'walmart',
#other
    'netflix',
    'skype',
    'github',
    'onedrive',
    'dropbox',
#crytocurrency
    'blockchain',
}

popular_keywords = {
    'login',
    'log-in',
    'sign-in',
    'signin',
    'account',
    'verification',
    'verify',
    'webscr',
    'password',
    'credential',
    'support',
    'activity',
    'security',
    'update',
    'authentication',
    'authenticate',
    'authorize',
    'wallet',
    'alert',
    'purchase',
    'transaction',
    'recover',
    'unlock',
    'confirm',
    'live',
    'office',
    'service',
    'manage',
    'portal',
    'invoice',
    'secure',
    'customer',
    'client',
    'bill',
    'online',
    'safe',
    'form',
    'bank',
    'reactivate'
}

tlds = {
    'ga',
    'gq',
    'ml',
    'cf',
    'tk',
    'xyz',
    'pw',
    'cc',
    'club',
    'work',
    'top',
    'support',
    'bank',
    'info',
    'study',
    'click',
    'country',
    'stream',
    'gdn',
    'mom',
    'xin',
    'kim',
    'men',
    'loan',
    'download',
    'racing',
    'online',
    'center',
    'ren',
    'gb',
    'win',
    'review',
    'vip',
    'party',
    'tech',
    'science',
    'business'
}




import vt_query_host as vtq
import sys
import tldextract
import entropy

def get_set(filename):
    words = set()
    fn = open(filename)
    for word in fn:
        words.add(word.strip())
    return words

def ext_features(infilename, outfilename, keywordfile):
    infile = open(infilename)
    outfile = open(outfilename, "w+")

    keywords = get_set(keywordfile)

    for domain in infile:
        domain = domain.strip()
        record = vtq.get_record(domain, "vt_nod")
        if record == None:
            continue
        lifetime = record[1] - record[0]
        total_count = record[2]    
        mal_count = record[3]
        ratio = float(mal_count)/total_count
        fqdns = record[4].split('##')
        no_fqdns = len(fqdns)
        keyword_count = 0
        dot_count = 0
        sentropy = 0.0
        for fqdn in fqdns:
            ext = tldextract.extract(fqdn)
            if ext.subdomain != None and len(ext.subdomain) > 0:
                dot_count += ext.subdomain.count('.')
                if ext.subdomain in keywords:
                    keyword_count += 1
                sentropy += entropy.shannon_entropy(ext.subdomain)
        outfile.write("{} {} {} {} {} {} {} {} {} {}\n".format(domain, lifetime, total_count, mal_count, ratio, no_fqdns, keyword_count, float(keyword_count)/no_fqdns, float(dot_count)/no_fqdns, float(sentropy)/no_fqdns))
    outfile.close()

def subdomains(infilename, outfilename, threshold):
    infile = open(infilename)
    outfile = open(outfilename, "w+")

    sub_freq = dict()
    for domain in infile:
        domain = domain.strip()
        record = vtq.get_record(domain, "vt_nod")
        if record == None:
            continue
        fqdns = record[4].split('##')
        for fqdn in fqdns:
            ext = tldextract.extract(fqdn)
            if ext.subdomain == None or len(ext.subdomain) == 0:
                continue
            if '.' in ext.subdomain:
                continue
            if ext.subdomain not in sub_freq:
                sub_freq[ext.subdomain] = 0
            sub_freq[ext.subdomain] += 1
    print(sub_freq)
    for subdom, freq in sub_freq.items():
        if freq > threshold:
            outfile.write("{} {}\n".format(subdom, freq))
    outfile.close()

if __name__ == "__main__":
    if len(sys.argv) < 4:
        print("{} input-domain-file output-feature-file top-keywords-file".format(sys.argv[0]))
        exit()
    ext_features(sys.argv[1], sys.argv[2], sys.argv[3])



import glob
import json
import pandas as pd

def update_labels(fn, labels):
    df1 = pd.read_csv(fn, header = None)
    df1.columns = ['label']
    current_labels = set(df1['label'].unique())
    all_labels = current_labels | set(labels)
    df = pd.DataFrame(all_labels)
    df.to_csv(fn, header = False, index = False)
    return all_labels

all_attacks = update_labels("/export/sec02/nabeel/Common/utils/vt_attack_universe.txt", [])

def vt_json_to_dict(line, attacks):
    record = dict()
    try:
        jobj = json.loads(line)
        url = jobj["url"].strip("/")
        record["url"] = url
        record["scan_date"] = jobj["scan_date"]
        record["positives"] = jobj["positives"]
        record["first_seen"] = jobj["first_seen"]
        record["last_seen"] = jobj["last_seen"]
        
        #additional information
        record["ip"] = "NONE"
        try:
            record["ip"] = jobj["additional_info"]["resolution"]
        except KeyError as e:
            pass
        record["content_hash"] = "NONE"
        try:
            record["content_hash"] = jobj["additional_info"]["Response content SHA-256"]
        except KeyError as e:
            pass
        record["rlength"] = -1
        try:
            record["rlength"] = jobj["additional_info"]["rlength"]
        except KeyError as e:
            pass
        
        record["url_after_redirects"] = "NONE"
        try:
            record["url_after_redirects"] = jobj["additional_info"]["URL after redirects"]
        except KeyError as e:
            pass
        
        for attack in attacks:
            record[attack] = 0
        
        scans = jobj["scans"]
        record["avs"] = ""
        for key, value in scans.items():
            try:
                #we keep track of only those attack types we identified
                #ignore those not on the list through this exception block
                record[value['result']] += 1
            except KeyError as e:
                pass
            if value["detected"] == True:
                if len(record["avs"]) > 0:
                    record["avs"] += '#'
                record["avs"] += key + ':' + value['result']
    except KeyError as e:
        #print(e)
        pass
    return record

#consider multiple files matching a regular expression
def vt2df(filepath, attacks):
    records = []
    for filename in glob.iglob(filepath):
        fp = open(filename, encoding = 'utf-8')
        for line in fp:
            if len(line.strip()) == 0:
                continue
            records.append(vt_json_to_dict(line.strip(), attacks))
    return pd.DataFrame(records)

#consider only one file
def vt2df_file(filename, attacks):
    records = []
    fp = open(filename, encoding = 'utf-8')
    for line in fp:
        if len(line.strip()) == 0:
            continue
        records.append(vt_json_to_dict(line.strip(), attacks))
    return pd.DataFrame(records)



'''
Description: Collect reverse DNS records related data for IPs
'''
import sys
from multiprocessing import Pool
import socket

#given an IP, get the PTR record
def get_rdns(ip):
  records = list()
  try:
    answer = socket.gethostbyaddr(ip)
    if len(answer) > 1:
        records.append("{} {}".format(ip, answer[0]))
  except socket.herror as e:
    answer = "##UNKNOWNHOST##"
    records.append("{} {}".format(ip, answer))
  except Exception as e:
    answer = "##ERRORR##"
    records.append("{} {}".format(ip, answer))
    print(e)
  return records

#Get DNS detaisl for a list of domains in a text file and write the output back
#to a file
def get_rdns_file(in_put):
    records = get_rdns(in_put)
    return records


#run the program
if __name__ == '__main__':
    inputf  = open(sys.argv[1].strip(),'r')
    outputf = open(sys.argv[2].strip(),'w+')
    #use 30 cores
    workers = Pool(30)
    #parallel workload -asynchronous
    records = workers.map(get_rdns_file, [x.split('\n')[0] for x in inputf.readlines()])
    #write on file - synchronous
    for result in records:
        for record in result:
            outputf.write(str(record)+'\n')
        
    inputf.close()
    outputf.close()    



"""
Get the firstseen records from phishtank and analyze
"""
import copy
import sys
from datetime import datetime

import pandas as pd
import psycopg2

from commons.utils.domain_tools import get_2ld, get_fqdn, load_pub_types, get_pub_type, get_protocol

# table schema
# create table phishtank (url text primary key, domain varchar(500), phish_detail_url varchar(1000), first_submitted timestamp, last_submitted timestamp, no_submissions integer, first_verified tiimestamp, last_verified timestamp, verified text, online text, target text);

databasename = "phishing"
tablename = "phishtank"


def db_conn():
    conn = None
    try:
        conn = psycopg2.connect(database=databasename, user="postgres", password="postgres", port=5432)
    except:
        pass
    return conn


def db_close(conn):
    conn.close()


def get_firstseen(timestamp_file):
    conn = db_conn()
    urls = []
    if conn == None:
        print("Unable to connect to the DB")
        return urls

    cursor = conn.cursor()
    ts_str = open(timestamp_file).read().strip()
    ts_dt = datetime.strptime(ts_str, "%Y-%m-%d %H:%M:%S")
    query = "SELECT url, first_submitted FROM {} WHERE first_submitted > %s".format(tablename)
    params = (ts_str,)
    cursor.execute(query, params)
    rs = cursor.fetchall()
    first_submitted = ts_dt
    if rs != None:
        for record in rs:
            if first_submitted < record[1]:
                first_submitted = record[1]
            urls.append(record[0])
    fh = open(timestamp_file, "w")
    fh.write("{}".format(first_submitted.strftime("%Y-%m-%d %H:%M:%S")))
    fh.close()
    cursor.close()
    db_close(conn)
    return urls


def enumerate_urls(urls):
    pubs = load_pub_types()
    urls = set([url.strip('"/') for url in urls])
    new_urls = copy.deepcopy(urls)
    for url in urls:
        protocol = get_protocol(url)
        fqdn = get_fqdn(url)
        apex = get_2ld(fqdn)
        urlprime = url
        if protocol == "https":
            urlprime = urlprime.replace("https", "http", 1)
        elif protocol == "http":
            urlprime = urlprime.replace("http", "https", 1)
        # if it a protocol other than http/https, ignore
        if urlprime != url:
            new_urls.add(urlprime)

        # if protocol is not specified, use https as attackers
        # are increasingly forced to use https to avoid suspision
        if protocol == None:
            protocol = "https"

        if fqdn != None:
            fqdn = protocol + "://" + fqdn
            new_urls.add(fqdn)

        # only add non-public apex domains
        if apex != None:
            pub_type = get_pub_type(apex, pubs)
            if pub_type == "NotFound":
                apex = protocol + "://" + apex
                new_urls.add(apex)
    return new_urls


def save_urls(filename, urls):
    df = pd.DataFrame(urls)
    df.to_csv(filename, header=False, index=False, sep=" ")


if __name__ == "__main__":
    conn = db_conn()
    urls = get_firstseen(sys.argv[1])
    new_urls = enumerate_urls(urls)
    save_urls(sys.argv[2], new_urls)



'''
Gather VT IP data
'''
import vt_private as vtp
import sys
import json
from urllib.parse import urlparse
import time

SPEED = 0.1
def cert_file(ip_file, outfile, relation, keyindex):
    of = open(outfile, "w+")
    with open(ip_file) as df:
        for ip in df:
            time.sleep(SPEED)
            ip = ip.strip()
            of.write("{}\n".format(json.dumps(vtp.getIP(ip, relation, keyindex))))
    of.close()

if __name__ == "__main__":
    if len(sys.argv) < 5:
        print("Usage: {} <ip-file> <output-file> <relationship> <keyindex>".format(sys.argv[0]))
        exit()
    ipfile = sys.argv[1]
    outputfile = sys.argv[2]
    #historical_ssl_certificates,historical_whois, resolutions
    relation = sys.argv[3]
    keyindex = int(sys.argv[4])

    cert_file(ipfile, outputfile, relation, keyindex)



import os
from datetime import datetime

os.environ["TZ"] = "UTC"

# get the current date in YYYYMMDD format
# note: no validation
def get_current_date():
    return str(datetime.now().strftime("%Y%m%d"))

def get_current_time_epoch():
    return time.mktime(datetime.now().timetuple())

# get the date object for a date string of the format YYYYMMDD
# note that no validation is performed here; so make sure to
# pass the date in the correct format
def get_date_obj(date_str):
    return datetime.strptime(date_str, "%Y%m%d").date()

def validate_date(date_text):
    try:
        get_date_obj(date_text)
    except ValueError:
        raise ValueError("Invalid date format, should be YYYY-MM-DD")

def date2epoch(datestr, dformat = '%Y-%m-%dT%H:%M:%SZ'): #dformat = '%Y%m%d %H:%M'):
    return int(datetime.strptime(datestr, dformat).strftime('%s'))

def epoch2datestr(ts_epoch, dformat = '%Y%m%d %H:%M'):
    return datetime.utcfromtimestamp(ts_epoch).strftime(dformat)



'''
Description: Collect active DNS records related data for domains
'''
import dns.resolver
import sys
from multiprocessing import Pool

#given a domain, get the DNS records
def get_dns_resolution(domain):
  records = list()
  try:
    answer = dns.resolver.query(domain)
    records.append(answer.rrset)
    records.append("{}. canonicalname {}".format(domain, answer.canonical_name))
    records.append("{}. qname {}".format(domain, answer.qname))
    records.append("{}. expiration {}".format(domain, answer.expiration))
    records.append("{}. rdclass {}".format(domain, answer.rdclass))
    records.append("{}. rdtype {}".format(domain, answer.rdtype))
    try:    
      mxrecords = dns.resolver.query(domain, 'MX')
      for rdata in mxrecords:
        records.append("{}. MX {}:{}".format(domain, rdata.exchange, rdata.preference))
    except dns.resolver.NoAnswer:
      pass #ignore ones without MX records
    try:
      nsrecords = dns.resolver.query(domain, 'NS')
      for rdata in mxrecords:
        records.append("{}. NS {}".format(domain, rdata))
    except:
      pass #ignore ones without NS records
  except dns.resolver.NXDOMAIN as e:
    answer = "##NXDOMAIN##"
    records.append("{}. {}".format(domain, answer))
  except dns.resolver.Timeout:
    answer = "##TIMEOUT##"
    records.append("{}. {}".format(domain, answer))
  except Exception as e:
    answer = "##ERRORR##"
    records.append("{}. {}".format(domain, answer))
    print(e)
  return records

#Get DNS detaisl for a list of domains in a text file and write the output back
#to a file
def get_dns_resolutions_file(in_put):
    records = get_dns_resolution(in_put)
    return records


#run the program
if __name__ == '__main__':
    inputf  = open(sys.argv[1].strip(),'r', encoding = 'utf-8')
    outputf = open(sys.argv[2].strip(),'w+', encoding = 'utf-8')
    #use 30 cores
    workers = Pool(30)
    #parallel workload -asynchronous
    records = workers.map(get_dns_resolutions_file, [x.split('\n')[0] for x in inputf.readlines()])
    #write on file - synchronous
    for result in records:
        for record in result:
            outputf.write(str(record)+'\n')
        
    inputf.close()
    outputf.close()    



import sys
import os

def download(url, phishid, folder):
    os.system("mkdir -p {}".format(folder))
    os.system("curl -O {}".format(url))
    os.system("mv {}.jpg {}".format(phishid, folder))

def download_file(filename, folder):
    f = open(filename, encoding = 'utf-8')
    for line in f:
        try:
            arr = line.split("#####")
            if arr == 0:
                continue
            download("https://d1750zhbc38ec0.cloudfront.net/{}.jpg".format(arr[0].strip()), arr[0].strip(), folder)
        except:
            print("Download error {}".format(line))
            pass

download_file(sys.argv[1], sys.argv[2])



import pandas as pd

public_df = pd.read_csv("/export/sec02/nabeel/Common/pub/public_apex_domains_20201127", header = None)
public_df.columns = ['domain']
public_domains = set(public_df['domain'].unique())

def is_public(apex):
    global public_domains
    if apex in public_domains:
        return True
    return False



import requests
import io
import tarfile
import json
import sys
import os

#Note: this is the private key. We have a rate limit of 1000 queries per day.
#Please use this sparingly as we use this quota to download VT URL feed daily as well 
#APIKEY = '9e9ce9e13e321609fa73ed57d287ccae92986a21647d70c720f1cb9b9ef86c31'

#nabeel academic - valid from 8/5/2020 to 7/11/2020 (6 months)
#20K queries per day
APIKEY = '0b48e71dc148ab5935780bf74b07cadc7049b6684ea4d0bcffbaf5070db4c6bf'

VT_URL_FEED_URL = "https://www.virustotal.com/vtapi/v2/url/feed"

#pkg format is YYYYMMDDTHH
def get_file(pkg, folder):
    querystring = {"apikey": APIKEY, "package": pkg}
    outf = os.path.join(folder, "url-{0}.tar.bz2".format(pkg))
    try:
        response = requests.get(VT_URL_FEED_URL, params=querystring)
        print(len(response.content))
        package_file = open(outf, 'wb')
        package_file.write(response.content)
        package_file.close()
    except Exception as e:
        print(e)
        print("failed: {}".format(pkg))

get_file(sys.argv[1], sys.argv[2])



'''
Description: Given an IP address, it finds the cloud hosting provider (if any). 
Currently, we support aws, gce and azure.
Author: nabeelxy
'''
import ipaddress

class IP2Cloud:
    #filelist - a csv file of the format <cloud>,<filepath>
    def __init__(self, filelist):
        file_dict = dict()
        fh = open(filelist)
        for line in fh:
            arr = line.split(",")
            if len(arr) != 2:
                raise ValueError("ERROR: invalid format: {}".format(line))
            cloud = arr[0].strip()
            filename = arr[1].strip()
            file_dict[cloud] = filename
        self.clouds = dict()
        for cloud in file_dict:
            self.clouds[cloud] = self.load_subnets(file_dict[cloud])

    def load_subnets(self, inputfile):
        ipf = open(inputfile)
        subnets = dict()
        for subnet in ipf:
            subnet = subnet.strip()
            n = ipaddress.ip_network(subnet)
            subnets[subnet] = [int(n.network_address), int(n.netmask)]        
        return subnets

    def in_network(self, address, network, mask):
        addr = int(ipaddress.ip_address(address))
        return (addr & mask) == network

    def get(self, address):
        for cloud in self.clouds:
            for subnetname in self.clouds[cloud]:
                subnet = self.clouds[cloud][subnetname]
                if self.in_network(address, subnet[0], subnet[1]):
                    return cloud
        return None
    
        



#!/usr/bin/python
'''
Author: @nabeelxy
Description:
This file is specifically prepared for phicious project
Given a list of domains in a file, it produces a file with the following features:
'''

import sys
import pdns
import tldextract

def get_2ld(fqdn):
    ext = tldextract.extract(fqdn)
    return ext.domain + "." + ext.suffix

def get_pdns_features(domain, firstseen_before = None):
    first_seen = None
    last_seen = None
    count = 0
    ip_set = set()
    first_seen_date = None
    last_seen_date = None
    LAST_SEEN_AFTER = "20150601 00:00"
    mappings = pdns.pdns_domain(domain, firstseen_before, None, None, LAST_SEEN_AFTER, 5000)

    if mappings == None:
        return None
    
    for entry in mappings:
        for ip in entry["rdata"]:
            ip_set.add(ip)
    return ip_set

def get_features_file(domain_file, outfile, firstseen_before = None):
    df = open(domain_file)
    of = open(outfile, "w+")
    for domain in df:
        domain = domain.strip()
        ip_set = get_pdns_features(domain, firstseen_before)
        if ip_set != None:
            for ip in ip_set:
                of.write("{} {}\n".format(domain, ip))
    of.close()  
    df.close() 

if __name__ == "__main__":
    if len(sys.argv) < 4:
        print("usage: python {} input_domain_file output_file firstseen_before(YYYYMMDD HH:MM)".format(sys.argv[0]))
        exit()
        
    domainf = sys.argv[1] #input domain  file
    outf = sys.argv[2] #output file
    firstseen_before = sys.argv[3]
    if firstseen_before.lower() == "none":
        firstseen_before = None
    get_features_file(domainf, outf, firstseen_before)



import sys
import tldextract
import pandas as pd

def get_phishtank_data(inputfile, outfile):
    inf = open(inputfile)
    data = set()
    for line in inf:
        arr = line.split(',')
        if len(arr) < 2:
            continue
        data.add(arr[0])
        data.add(arr[1])
        ext = tldextract.extract(arr[1])
        data.add(ext.domain + "." + ext.suffix)
    df = pd.DataFrame(data)
    df.to_csv(outfile, header = None, index = False)

get_phishtank_data(sys.argv[1], sys.argv[2])    



import json
import os
import sys
import time
from datetime import datetime
from multiprocessing.pool import ThreadPool

from commons.proxy import proxies

'''
Description: Scan and collect VT results using public keys
We use a proxy implementation to do the scanning in parallel
Author: @nabeelxy
'''
VT_SCAN_URL = 'https://www.virustotal.com/vtapi/v2/url/scan'
VT_REPORT_URL = 'https://www.virustotal.com/vtapi/v2/url/report'


def load_keys(keyfile):
    keys = list()
    with open(keyfile) as kf:
        for line in kf:
            keys.append(line.strip())
    return keys


def scan_proxy(url, key, index):
    data = {'apikey': key, 'url': url}
    req = proxies.proxy_exe(VT_SCAN_URL, index, proxies.proxy_pnts, "post", None, data)
    if req == None:
        return {'ERROR': 'None Returned'}
    return req.json()


def report_proxy(resource, key, index):
    data = {'apikey': key, 'resource': resource}
    req = proxies.proxy_exe(VT_REPORT_URL, index, proxies.proxy_pnts, "post", None, data)
    if req == None:
        return {'ERROR': 'None Returned'}
    return req.json()


def scan_thread(domainlist, index, key, folder):
    results = list()
    of = open(os.path.join(folder, "part_" + str(index)), "w+")
    for domain in domainlist:
        domain = domain.strip()
        result = json.dumps(scan_proxy(domain, key, index))
        results.append(result)
        of.write("{}\n".format(result))
        time.sleep(16)  # VT public allows only 4 queries per minute
    of.close()
    print("Thread {} Done".format(index))
    return results


def report_thread(scanidlist, index, key, folder):
    results = list()
    of = open(os.path.join(folder, "part_" + str(index)), "w+")
    for scanid in scanidlist:
        result = json.dumps(report_proxy(scanid, key, index))
        of.write("{}\n".format(result))
        results.append(result)
        time.sleep(16)  # VT public allows only 4 queries per minute
    of.close()
    return results


# number of threads = number of keys
# assumes that number of proxies >= number of keys
# TODO: improvement - handle the case where the number of proxies are less than
# the number of keys - implement proxy rotation
def run_threads(inputfile, outputfile, keys, method):
    inputlist = list()
    if method == "scan":
        with open(inputfile) as df:
            for line in df:
                try:
                    inputlist.append(line.strip())
                except TypeError as e:
                    pass
    elif method == "report":
        with open(inputfile) as sf:
            for line in sf:
                json_obj = json.loads(line)
                try:
                    scan_id = json_obj['scan_id'].strip()
                    inputlist.append(scan_id)
                except KeyError as e:
                    pass
                except TypeError as e2:
                    pass
    else:
        print("Unsupported method {}".format(method))
        return

    num_threads = len(keys)
    block_size = int(len(inputlist) / num_threads)
    all_results = list()
    pool = ThreadPool(num_threads)
    folder = datetime.today().strftime('%Y%m%d%H%M')

    if not os.path.exists(folder):
        os.makedirs(folder)

    for t in range(num_threads):
        start = t * block_size
        end = (t + 1) * block_size
        if t == num_threads - 1:
            end = len(inputlist)
        print("starting proxy {} key {} proxy {} items {}".format(t, keys[t], proxies.proxy_pnts[t],
                                                                  len(inputlist[start:end])))
        if method == "scan":
            all_results += [pool.apply_async(scan_thread,
                                             kwds={'domainlist': inputlist[start:end], 'index': t, 'key': keys[t],
                                                   'folder': folder})]
        else:
            all_results += [pool.apply_async(report_thread,
                                             kwds={'scanidlist': inputlist[start:end], 'index': t, 'key': keys[t],
                                                   'folder': folder})]
    pool.close()
    pool.join()

    of = open(outputfile, "w+")
    for result in all_results:
        outputlist = result.get()
        for line in outputlist:
            of.write("{}\n".format(line))
    of.close()
    print("All Done")


if len(sys.argv) < 5:
    print("Usage: {} method [scan|report] key_file input_file output_file".format(sys.argv[0]))
    exit()

method = sys.argv[1]  # scan or report
keys = load_keys(sys.argv[2])
domainfile = sys.argv[3]
outputfile = sys.argv[4]
run_threads(domainfile, outputfile, keys, method)



import sys
import psycopg2
import myutils
import string

def get_count(cur, sequence):
  tot_rows = 0
  for char in sequence:
    letter_rows = 0
    for char2 in sequence:
      stmt = "SELECT count(*) FROM " + myutils.get_table(char + char2)
      cur.execute(stmt)
      row = cur.fetchone()
      row_count = int(row[0])
      print (stmt, " ", row_count)
      letter_rows += row_count
      
    stmt = "SELECT count(*) FROM " + myutils.get_table(char + "*")
    cur.execute(stmt)
    row = cur.fetchone()
    row_count = int(row[0])
    letter_rows += row_count
    print (stmt, " ", row_count)
    print ('Number of rows in {0} = {1}'.format(char, letter_rows))
    tot_rows += letter_rows
  return tot_rows

def count_rows(databasename):
  try:
    tot_rows = 0
    conn = psycopg2.connect(database=databasename, user="pdns", password="pdns", host="10.4.8.61", port=5432)
    cur = conn.cursor()
    tot_rows += get_count(cur, string.ascii_lowercase)
    tot_rows += get_count(cur, string.digits)
    stmt = "SELECT count(*) FROM " + myutils.get_table("*")
    cur.execute(stmt)
    row = cur.fetchone()
    row_count = int(row[0])
    tot_rows += row_count
    print ('Number of rows in {0} = {1}'.format("*", row_count))
    cur.close()
    print ('Total rows = {0}'.format(tot_rows))
  except Exception as e:
    print (e)
    print ("some error related to db occurred")
  finally:
    if conn is not None:
      conn.close()

if __name__=="__main__":
  if len(sys.argv) < 2:
    print("Usage: {} <database".format(sys.argv[0]))
    exit()
  database = sys.argv[1]
  count_rows(database)



"""
Description: Scanning URLs in urlscan.io and obtaining report, screenshot and dom
"""
import json
import shutil
import sys
import time
from urllib.parse import urlparse

import requests

SPEED_SCAN = 8
SPEED_REPORT = 1


def get_fqdn(url):
    try:
        uri = urlparse(url)
        if uri:
            return uri.netloc
    except Exception as e:
        pass


def scan_url(url, key):
    try:
        headers = {'API-Key': key, 'Content-Type': 'application/json'}
        data = {"url": url, "visibility": "public"}
        response = requests.post('https://urlscan.io/api/v1/scan/', headers=headers, data=json.dumps(data))
        if response != None:
            return response.json()
        else:
            return {"ERROR": url}
    except requests.ConnectTimeout as e:
        return {'ERROR': '##timed out', 'url': url}


def get_report(url, key):
    try:
        headers = {'API-Key': key, 'Content-Type': 'application/json'}
        response = requests.get(url, headers=headers)
        if response:
            return response.json()
        else:
            return {"ERROR": url}
    except requests.ConnectTimeout as e:
        return {'ERROR': '##timed out', 'url': url}


# curl https://urlscan.io/screenshots/$uuid.png
def get_screenshot(uuid, key, folder):
    url = "https://urlscan.io/screenshots/{}.png".format(uuid)
    headers = {'API-Key': key}
    try:
        response = requests.get(url, stream=True, headers=headers)
        with open('{}/{}.png'.format(folder, uuid), 'wb') as out_file:
            shutil.copyfileobj(response.raw, out_file)
    except Exception as e:
        pass


# curl https://urlscan.io/dom/$uuid/
def get_dom(uuid, key, folder):
    url = "https://urlscan.io/dom/{}/".format(uuid)
    headers = {'API-Key': key}
    try:
        response = requests.get(url, headers=headers)
        with open('{}/{}.html'.format(folder, uuid), 'wb') as out_file:
            out_file.write(response.content)
    except Exception as e:
        pass


def scan_file(domain_file, outfile, key):
    of = open(outfile, "w+", encoding='utf-8')
    with open(domain_file, encoding='utf-8') as df:
        for line in df:
            time.sleep(SPEED_SCAN)
            line = line.strip()
            if line.startswith("http"):
                domain = line
            else:
                domain = "https://" + line
            try:
                of.write("{}\n".format(json.dumps(scan_url(domain, key))))
            except Exception as e:
                pass
    of.close()


def report_file(scanid_file, outfile, key, outfolder):
    of = open(outfile, "w+", encoding='utf-8')
    with open(scanid_file, encoding='utf-8') as sf:
        for str in sf:
            json_obj = json.loads(str)
            try:
                time.sleep(SPEED_REPORT)
                if json_obj["message"] == "Submission successful":
                    of.write("{}\n".format(json.dumps(get_report(json_obj["api"], key))))
                get_screenshot(json_obj["uuid"], key, outfolder)
                get_dom(json_obj["uuid"], key, outfolder)
            except KeyError as e:
                pass
    of.close()


def get_scanned_domains(infile, outfile):
    of = open(outfile, "w+", encoding='utf-8')
    with open(infile, encoding='utf-8') as sf:
        for str in sf:
            json_obj = json.loads(str)
            try:
                if json_obj["message"] == "Submission successful":
                    of.write("{}\n".format(get_fqdn(json_obj["url"])))
                elif json_obj["message"] == "DNS Error - Could not resolve domain":
                    of.write("{}\n".format(json_obj["description"].split()[2]))
            except KeyError as e:
                pass
    of.close()


if __name__ == "__main__":
    option = sys.argv[1]
    if option == "scan":
        # url file, #scan output file, #key index
        scan_file(sys.argv[2], sys.argv[3], 0)
    elif option == "report":
        # scan output file, report output file, key index, output folder
        report_file(sys.argv[2], sys.argv[3], 0, sys.argv[4])
    elif option == "both":
        # url file, #scan output file, #report key index, ouput file
        scan_file(sys.argv[2], sys.argv[3], 0)
        report_file(sys.argv[3], sys.argv[4], 0, sys.argv[5])
    elif option == "scanned_domains":
        get_scanned_domains(sys.argv[2], sys.argv[3])

# get_screenshot("27f74f26-a5a4-4199-90ad-2608b3394545", 0, "images")
# get_dom("27f74f26-a5a4-4199-90ad-2608b3394545", 0, "html")



'''
Description: utility functions
Author: @nabeelxy 
'''
from datetime import datetime
import tldextract
import time

# get the current date in YYYYMMDD format
# note: no validation
def get_current_date():
 return str(datetime.now().strftime("%Y%m%d"))

def get_current_time_epoch():
  return time.mktime(datetime.now().timetuple())
# get the date object for a date string of the format YYYYMMDD
# note that no validation is performed here; so make sure to
# pass the date in the correct format
def get_date_obj(date_str):
  return datetime.strptime(date_str, "%Y%m%d").date()

#get the TLD of a given FQDN (e.g. mail.qcri.org --> qcri.org, www.bbc.co.uk --> bbc.co.uk)
def get_2ld(fqdn):
  ext = tldextract.extract(fqdn)
  return ext.domain + "." + ext.suffix

def get_all_tlds(domain_file, outfile):
    twolds = set()
    with open(domain_file) as df:
        for domain in df:
            domain = domain.strip()
            twolds.add(get_2ld(domain))
    of = open(outfile, "w+")
    for twold in twolds:
        of.write("{}\n".format(twold))
    of.close()
